package com.example.kamizelka;

import android.app.Activity;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothProfile;
import android.content.Intent;
import android.util.Log;
import android.util.Pair;

import java.util.ArrayList;
import java.util.List;

public class BluetoothLEServiceConnecter extends BluetoothLeService{
    private int addressIterator = 0;
    private BluetoothGatt bluetoothGatt;
    private final ArrayList<DeviceHolder> deviceHolders = new ArrayList<>();
    private final ArrayList<Pair<BluetoothGattCharacteristic, ArrayList<BluetoothGattDescriptor>>> needToReadCharacteristics = new ArrayList<>();
    protected DeviceListDialog dialogfragment;
    public ArrayList<String> remoteDeviceAddresses = new ArrayList<>();

    protected final static String TAG = "BLUETOOTH CONNECTER";

    public BluetoothLEServiceConnecter(Activity active, DeviceListDialog deviceListDialog) {
        super(active);
        this.dialogfragment = deviceListDialog;
    }

    public final BluetoothGattCallback gattCallback = new BluetoothGattCallback() {                             // Implements callback methods for GATT events that the app cares about.
        @Override
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {                     // Вызывается при подключении/отключении соединения с сервером.
            if (newState == BluetoothProfile.STATE_CONNECTED) {
                Log.i(TAG, "Connected to GATT server.");
                ++addressIterator;
                if(bluetoothGatt.requestMtu(Defines.MTU_VALUE)) Log.i(TAG, "MTU: SET!"); else Log.i(TAG, "MTU: NOT SET!");
            } else if (newState == BluetoothProfile.STATE_DISCONNECTED) {
                Log.i(TAG, "Disconnected from GATT server.");
//                if(dialogfragment != null)
//                    dialogfragment.dismiss();
                if (addressIterator < remoteDeviceAddresses.size()) {
                    close();
                    connect(remoteDeviceAddresses.get(addressIterator));                                        // Если ещё остались неопрошенные устройства, опрашиваем.
                    return;
                }else {
                    for(int i = 0; i < deviceHolders.size(); ++i){
                        for(int j = i + 1; j < deviceHolders.size(); ++j){
                            if(deviceHolders.get(j).serialNumber.equals(deviceHolders.get(i).serialNumber)){    // Если у двух устройств тот же самый серийный номер, то это правый и левый каналы.
                                //dialogfragment.addItem(deviceHolders.get(i).serialNumber);
                                if(deviceHolders.get(j).channel.equals("Right") ){                              // Делаем из вдух устройств одно двухканальное.
                                    deviceHolders.get(j).addressLeft = deviceHolders.get(i).addressRight;
                                    deviceHolders.remove(i);
                                    --i;
                                }else if(deviceHolders.get(i).channel.equals("Right")){
                                    deviceHolders.get(i).addressLeft = deviceHolders.get(j).addressRight;
                                    deviceHolders.remove(j);
                                }else
 //                                   dialogfragment.addItem(deviceHolders.get(i).serialNumber + " MONO", deviceHolders.get(i));    // Если второй канал не найден, значит это моно устройство.
                                Log.i(TAG, "Mono Device found!");
                            }}}
                    for(DeviceHolder dev: deviceHolders){                                                          // Добавляем серийный номер в список отображаемых устройств.
                        if(dev.addressLeft == null)
                            dialogfragment.addItem(dev.serialNumber, dev);
//                        dialogfragment.addItem(dev.serialNumber + " MONO", dev);
                        else
                            dialogfragment.addItem(dev.serialNumber, dev);
                    }
                    addressIterator = 0;
                    dialogfragment.show();
                }
                close();
            }
        }
        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            Log.w(TAG, "onServicesDiscovered received: " + status);
            List<BluetoothGattService> services = bluetoothGatt.getServices();
            boolean b = false;
            for (BluetoothGattService service : services) {
                Log.w(TAG, "Stream Line "+ service.getUuid().toString());
                if (service.getUuid().toString().equalsIgnoreCase(DeviceHolder.dataServiceUUID)) {          // Если у удалённого устройства нет нужного сервиса, оно нам не интересно.
                    Log.w(TAG, "FOUND "+ service.getUuid().toString());
                    b = true;
                    break;
                }}
            if (b == false) remoteDeviceAddresses.remove(bluetoothGatt.getDevice().getAddress());
            else {
                for (int i = 0; i < services.size(); ++i) {
                    Log.w(TAG, "INSPECTED: "+ services.get(i).getUuid().toString());
                    for (BluetoothGattCharacteristic characteristic : services.get(i).getCharacteristics()) {       // Проходим по всем харрактеристикам.
                        Log.w(TAG, "CHARACTERISTIC ADDED: "+ characteristic.getUuid().toString());
                        ArrayList<BluetoothGattDescriptor> descriptors = new ArrayList<>();
                        for(BluetoothGattDescriptor descr: characteristic.getDescriptors()){
                            Log.w(TAG, "DESCRIPTOR ADDED: "+ descr.getUuid().toString());
                            descriptors.add(descr);
                        }
                        Pair<BluetoothGattCharacteristic, ArrayList<BluetoothGattDescriptor>> p = new Pair<>(characteristic, descriptors);
                        needToReadCharacteristics.add(p);
                    }
                }
                Log.w(TAG, "frag");
                readWhatHaveToBeRead();
            }
        }
        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            Log.d(TAG, "onCharacteristicRead " + characteristic.getUuid().toString() + "\n");
            boolean b = false;
            DeviceHolder dev;
            int devnum = 0;
            for(DeviceHolder d: deviceHolders){
                if(d.addressRight.equals(bluetoothGatt.getDevice().getAddress())){
                    b = true;
                    devnum = deviceHolders.indexOf(d);
                    break;
                }}
            if(b == false){
                dev = new DeviceHolder(bluetoothGatt.getDevice().getAddress());
                deviceHolders.add(dev);
            }else
                dev = deviceHolders.get(devnum);
            dev.setValues(characteristic.getUuid().toString(), characteristic.getValue());
            readWhatHaveToBeRead();
        }
        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            Log.d(TAG, "onCharacteristicWrite");
//            bluetoothGatt.writeCharacteristic(charOfInterest);
        }
        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            Log.d(TAG, "onCharacteristicChanged");
        }
        @Override
        public void onDescriptorRead(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            Log.d(TAG, "onDescriptionRead " + descriptor.getUuid().toString() + "\n");
            readWhatHaveToBeRead();
        }
        @Override
        public void onMtuChanged(BluetoothGatt gatt, int mtu, int status) {
            Log.d(TAG, "onMTUChanged " + mtu + ", " + status + "\n");
            Log.i(TAG, "Attempting to start service discovery:");   // Attempts to discover services after successful connection.
            bluetoothGatt.discoverServices();
        }
    };

    private void readWhatHaveToBeRead(){
        Log.d(TAG, "needToReadCharacteristics size: "+needToReadCharacteristics.size() + "\n");
        if(!needToReadCharacteristics.isEmpty()) {                                                      // Ассинхронное чтение харрактеристик.
            if(!needToReadCharacteristics.get(needToReadCharacteristics.size()-1).second.isEmpty()){    // Читаются сначала дескрипторы, затем сами харрактеристики.
                BluetoothGattDescriptor descr = needToReadCharacteristics.get(needToReadCharacteristics.size()-1).second.get(
                        needToReadCharacteristics.get(needToReadCharacteristics.size()-1).second.size()-1);
                Log.d(TAG, "discr perm: "+descr.getPermissions() + "\n");
                readDescriptor(descr);
                needToReadCharacteristics.get(needToReadCharacteristics.size()-1).second.remove(needToReadCharacteristics.get(needToReadCharacteristics.size()-1).second.size()-1);
            }else{
                BluetoothGattCharacteristic c = needToReadCharacteristics.get(needToReadCharacteristics.size()-1).first;
                needToReadCharacteristics.remove(needToReadCharacteristics.size()-1);
                Log.d(TAG, "char perm: "+c.getProperties() + "\n");
                Log.d(TAG, "cRESULT: "+(c.getProperties() & BluetoothGattCharacteristic.PROPERTY_READ) + "\n");
                if((c.getProperties() & BluetoothGattCharacteristic.PROPERTY_READ) == BluetoothGattCharacteristic.PROPERTY_READ){
                    readCharacteristic(c);
                }else{
                    readWhatHaveToBeRead();
                }
            }}else {
                bluetoothGatt.disconnect();
        }
    }

    public void readCharacteristic(BluetoothGattCharacteristic characteristic) {
        Log.d(TAG, "read char");
        if (bluetoothAdapter == null || bluetoothGatt == null) {
            Log.w(TAG, "BluetoothAdapter not initialized");
            return;
        }
        bluetoothGatt.readCharacteristic(characteristic);
    }

    public void readDescriptor(BluetoothGattDescriptor descriptor) {
        Log.d(TAG, "read descr");
        if (bluetoothAdapter == null || bluetoothGatt == null) {
            Log.w(TAG, "BluetoothAdapter not initialized");
            return;
        }
        bluetoothGatt.readDescriptor(descriptor);
    }

    public void scanReady() {
        streamlineAddresses();
    }

    public boolean connect(final String address) {
        Log.w(TAG, "Connect to: " + address + "\n");
        if (bluetoothGatt != null && address.equals(bluetoothGatt.getDevice().getAddress())) {    // Если уже есть такое соединение.
            return bluetoothGatt.connect();
        }
        final BluetoothDevice device = bluetoothAdapter.getRemoteDevice(address);
        if (device == null) {
            Log.w(TAG, "Device not found.  Unable to connect.");
            return false;
        }
        bluetoothGatt = device.connectGatt(this, false, gattCallback);      // We want to directly connect to the device, so we are setting the autoConnect parameter to false.
        return true;
    }

    private void streamlineAddresses(){
        if(!remoteDeviceAddresses.isEmpty()){
            Log.w(TAG, "Number of remote devices: " + remoteDeviceAddresses.size() +"\n");
            addressIterator = 0;
            connect(remoteDeviceAddresses.get(0));
        }else {
            NoFoundDialog d = new NoFoundDialog(this.dialogfragment.getfManager());
            this.dialogfragment.dismiss();
            d.show();
            Log.w(TAG, "Nothing found.\n");
        }
    }

    public void delete() throws Throwable {
        if (bluetoothGatt == null)
            return;
        bluetoothGatt.close();
        this.finalize();
    }
    public void close() {
        Log.d(TAG, "close");
        if (bluetoothGatt == null)
            return;
        bluetoothGatt.close();
        bluetoothGatt = null;
    }

    public void disconnect() {
        Log.d(TAG, "disconnect");
        if (bluetoothAdapter == null || bluetoothGatt == null) {
            Log.w(TAG, "BluetoothAdapter not initialized");
            return;
        }
        bluetoothGatt.disconnect();
    }

    @Override
    public boolean onUnbind(Intent intent) {
        Log.d(TAG, "onUnbind");
        // After using a given device, you should make sure that BluetoothGatt.close() is called
        // such that resources are cleaned up properly.  In this particular example, close() is
        // invoked when the UI is disconnected from the Service.
        close();
        return super.onUnbind(intent);
    }
}
