package com.example.kamizelka;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothProfile;
import android.os.Build;
import android.os.Handler;
import android.os.Looper;
import android.os.SystemClock;
import android.util.Log;

import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

public class BluetoothLEServiceSender extends BluetoothLeService{
	ArrayList<GattHolder> gatts = new ArrayList<>();
	public ArrayList<String> addressesQueue = new ArrayList<>();
	protected boolean isConnectionOk = false;
	int nextStep = 0;
	protected final static String TAG = "BLUETOOTH SENDER";
	private long stopTime = SystemClock.uptimeMillis();
	private boolean isStoptimerWasSet = false;

	private SenderFunctional functional;
	private final SenderFunctionalFile fileFunc;
	private final SenderFunctionalMicrophone micFunc;
	private final SenderFunctionalKeyboard keyFunc;
	private final SenderFunctionStop stopFunc;

	enum Mode{
		file,
		microphone,
		keyboard,
		stopSound
	}

	public void setKeyboardSound(int index, byte value){
		this.keyFunc.setbytePWM(index, value);
	}

	@RequiresApi(api = Build.VERSION_CODES.M)
	public BluetoothLEServiceSender(AppCompatActivity active){
		super(active);
		this.addressesQueue.clear();
		this.fileFunc = new SenderFunctionalFile(active, this);
		this.micFunc = new SenderFunctionalMicrophone(active, this);
		this.keyFunc = new SenderFunctionalKeyboard(active, this);
		this.stopFunc = new SenderFunctionStop(active, this);
		this.functional = micFunc;
	}

	public void setFunctional(Mode m){
		switch (m){
			case file:
				this.functional = fileFunc;
				break;
			case microphone:
				this.functional = micFunc;
				break;
			case keyboard:
				this.functional = keyFunc;
				break;
			case stopSound:
				this.functional = stopFunc;
				break;
	}}

	public void startPlaying(Mode m){
		if(this.functional != null)
			this.functional.stopPlaying();
		setFunctional(m);
		this.functional.startPlaying();
	}

	public void stopPlaying(){
		this.functional.stopPlaying();
	}

	public boolean isConnectionOk(){
		Log.i(TAG, "connection is: " + isConnectionOk);
		return isConnectionOk;
	}

	public void mute(){
		if(SystemClock.uptimeMillis() < stopTime) {
			new Handler(Looper.getMainLooper()).postDelayed(new Runnable() {
				@Override
				public void run() {
					mute();
				}}, stopTime - SystemClock.uptimeMillis());
			return;
		}
		this.isStoptimerWasSet = false;
//		if(this.functional != this.keyFunc && this.functional.isPlaying == true)
		if(this.functional.isPlaying == true)
			return;
		startPlaying(Mode.stopSound);
	}

	public void setStop(){
		this.stopTime = SystemClock.uptimeMillis() + Defines.KEYBOARD_SOUND_DURATION_MS;
		if(this.isStoptimerWasSet == false){
				this.isStoptimerWasSet = true;
				new Handler(Looper.getMainLooper()).postDelayed(new Runnable() {
					@Override
					public void run() {
						mute();
					}}, Defines.KEYBOARD_SOUND_DURATION_MS);
		}
	}

	public void nextStep(){
		this.functional.nextStep();
	}

	public final BluetoothGattCallback gattCallback = new BluetoothGattCallback(){							 // Implements callback methods for GATT events that the app cares about.
		@RequiresApi(api = Build.VERSION_CODES.M)
		@Override
		public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState){					 // Вызывается при подключении/отключении соединения с сервером.
			if (newState == BluetoothProfile.STATE_CONNECTED){
				Log.i(TAG, "Connected to GATT server.");
				if(status != BluetoothGatt.GATT_SUCCESS){
					for(GattHolder g : gatts){
						if(g.gatt.getDevice().getAddress().equals(gatt.getDevice().getAddress())){
							g.setState(GattHolder.GattHolderState.notReady);
						}}
					functional.nextStep();
					return;
				}else if(gatt.requestMtu(Defines.MTU_VALUE))
					Log.i(TAG, "MTU: SET!");
				else
					Log.i(TAG, "MTU: NOT SET!");
			} else if (newState == BluetoothProfile.STATE_DISCONNECTED){
				Log.i(TAG, "Disconnected from GATT server." + gatt.getDevice().getAddress());
				for(GattHolder g : gatts){
					if(g.gatt.getDevice().getAddress().equals(gatt.getDevice().getAddress())){
						g.setState(GattHolder.GattHolderState.notReady);
					}} //9C:9C:1F:C4:EA:26
				functional.statusUpdate();
				functional.nextStep();
			}}
		@RequiresApi(api = Build.VERSION_CODES.M)
		@Override
		public void onMtuChanged(BluetoothGatt gatt, int mtu, int status){
			Log.d(TAG, "onMTUChanged " + mtu + ", " + status + "\n" + "Attempting to start service discovery:" + "\n");
			for(GattHolder g : gatts){
				if(g.gatt.getDevice().getAddress().equals(gatt.getDevice().getAddress())){
					Log.d(TAG, "Known device, will not read services again.\n");
					g.setState(GattHolder.GattHolderState.connected);
					functional.nextStep();
					return;
				}}
			gatt.discoverServices();
		}
		@RequiresApi(api = Build.VERSION_CODES.M)
		@Override
		public void onServicesDiscovered(BluetoothGatt gatt, int status) {
			Log.w(TAG, "onServicesDiscovered received, status: " + status);
			List<BluetoothGattService> services = gatt.getServices();
			for (BluetoothGattService service : services) {
				if (!service.getUuid().toString().equalsIgnoreCase(DeviceHolder.dataServiceUUID)){		  // Ищем сервис данных.
					continue;
				}
				Log.w(TAG, "service found! ");
				BluetoothGattCharacteristic ready = null;
				BluetoothGattCharacteristic data = null;
				BluetoothGattCharacteristic player = null;
				for (BluetoothGattCharacteristic characteristic : service.getCharacteristics()){		   // Нам нужны две его харрактеристики.
					if(characteristic.getUuid().toString().equals(DeviceHolder.readyCharacteristicUUID)){
						ready = characteristic;
						Log.w(TAG, "ready found! ");
					}
					if(characteristic.getUuid().toString().equals(DeviceHolder.dataCharacteristicUUID)){
						data = characteristic;
						Log.w(TAG, "data found! ");
					}
					if(characteristic.getUuid().toString().equals(DeviceHolder.playerCharacteristicUUID)){
						player = characteristic;
						Log.w(TAG, "data found! ");
					}
				}
				boolean exist = false;
				for (GattHolder g : gatts){
					if(gatt.getDevice().getAddress().equals(g.gatt.getDevice().getAddress()))
						exist = true;
				}
				if(!exist && data != null && ready != null) {							 // Если устройства ещё нет в списке и харрактеристики на месте, то сервис в порядке.
					GattHolder g = new GattHolder(gatt, data, ready, player);
					g.setState(GattHolder.GattHolderState.connected);
					gatts.add(g);														// Добавляем в список.
					Log.d(TAG, "gatts added, size: " + gatts.size());
				}
				functional.nextStep();
				return;
			}
		}
		@RequiresApi(api = Build.VERSION_CODES.M)
		@Override
		public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status){
			Log.d(TAG, "onCharacteristicRead " + characteristic.getUuid().toString() + "\n");
			if(characteristic.getUuid().toString().equals(DeviceHolder.readyCharacteristicUUID)){
				for(GattHolder g : gatts){
					if(g.gatt.getDevice().getAddress().equals(gatt.getDevice().getAddress())){
						String s = new BigInteger(1, characteristic.getValue()).toString(16);
						Log.d(TAG, "remote READY is: " + s + "\n");
						if(s.equals("1")) {
							Log.d(TAG, "It's ready!");
							g.state = GattHolder.GattHolderState.ready;
						}}}
				functional.statusUpdate();
			}
			functional.nextStep();
		}
		@RequiresApi(api = Build.VERSION_CODES.M)
		@Override
		public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status){
//			Log.d(TAG, "onCharacteristicWrite");
			if(characteristic.getUuid().toString().equals(DeviceHolder.playerCharacteristicUUID)){
				if(status == BluetoothGatt.GATT_SUCCESS) {
					Log.d(TAG, "Write self address, success.");
					for (GattHolder g : gatts) {
						if (g.gatt.getDevice().getAddress().equals(gatt.getDevice().getAddress())){
							g.setState(GattHolder.GattHolderState.written);
							break;
						}}}
				functional.nextStep();																			// Обновляем количество подключённых устройств.
				return;
			}

//			if(nextStep >= gatts.size() - 1){														// Если отправляем данные последнему устройству в списке, ставим будильник.
//				new Handler(Looper.getMainLooper()).postDelayed(new Runnable() {
//					@Override
//					public void run() {
//						functional.nextStep();
//					}}, 50);//Defines.SEND_PERIOD_MS * 2);
//				return;
//			}else
				functional.nextStep();
			// TODO здесь организуем задержку по времени для посылки данных.
		}
		@Override
		public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic){
			Log.d(TAG, "onCharacteristicChanged");
		}
		@Override
		public void onDescriptorRead(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status){
			Log.d(TAG, "onDescriptionRead " + descriptor.getUuid().toString() + "\n");
		}
	};

	public void makeNewConnection(){
		int i = 0;
		Log.d(TAG, "Make new connection, nextStep = " + nextStep);
		while(i < gatts.size()){
			if (gatts.get(i).gatt.getDevice().getAddress().equals(addressesQueue.get(nextStep)))	// Известное ли нам это устройство.
				break;
			++i;
		}
		if(i >= gatts.size()){																		// Если подключаемся в первый раз, создаём новый экземпляр BluetoothGatt.
			Log.d(TAG, "Unknown device, connected. nextStep = " + nextStep);
			if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M)
				bluetoothAdapter.getRemoteDevice(addressesQueue.get(nextStep)).connectGatt(this, false, gattCallback, BluetoothDevice.TRANSPORT_LE);
			else
				bluetoothAdapter.getRemoteDevice(addressesQueue.get(nextStep)).connectGatt(this, false, gattCallback);
		}else{																						// Если устройство известно,
			Log.d(TAG, "Known device, connected");
			if(gatts.get(i).reconnectCount < 20){													// Если подключались к устройству уже более 20 раз
				Log.d(TAG, "Reconnection count < 20, reconnected");
				gatts.remove(i);																	// Удаляем экземпляр BluetoothGatt и создаём новый.
				if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M)
					bluetoothAdapter.getRemoteDevice(addressesQueue.get(nextStep)).connectGatt(this, false, gattCallback, BluetoothDevice.TRANSPORT_LE);
				else
					bluetoothAdapter.getRemoteDevice(addressesQueue.get(nextStep)).connectGatt(this, false, gattCallback);
			}else {
				Log.d(TAG, "Reconnection count > 20, make new connection");
				++gatts.get(i).reconnectCount;														// Увеличиваем количество подключений.
				gatts.get(i).gatt.connect();														// Соеденяемся с устройством.
	}}}

	@RequiresApi(api = Build.VERSION_CODES.M)
	public void createConnections(ArrayList<String> addresses){
		this.addressesQueue = addresses;
		if(addressesQueue.isEmpty()){
			Log.d(TAG, "No addresses");
			return;
		}
		functional.nextStep();
	}

	public void delete() throws Throwable {
		for (GattHolder g : gatts){
			if (g.gatt == null) continue;
			g.gatt.close();
		}
		this.finalize();
	}

}
