package com.example.kamizelka;

import android.Manifest;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Button;
import android.widget.Toast;

public class MainActivity extends AppCompatActivity {
    private static final int REQUEST_MICROPHONE = 0;
    DeviceListDialog deviceListDialogfragment = null;
    OpenFileDialog openDialogfragment = null;
    BluetoothLEServiceConnecter connecter = null;
    BluetoothLEServiceScanner scanner = null;
    BluetoothLEServiceSender sender = null;
    ToneGenerator toneGenerator = new ToneGenerator();

    @Override
    public void onRequestPermissionsResult(int reqestCode, @NonNull String[] permissions, @NonNull int[] grantResults){
        super.onRequestPermissionsResult(reqestCode, permissions, grantResults);
        if(reqestCode == REQUEST_MICROPHONE){
            for (int i = 0; i < permissions.length; ++i) {
                if (grantResults[i] != PackageManager.PERMISSION_GRANTED)
                    this.finish();

    }}}
    private void aksPermission(String s){
        if (ContextCompat.checkSelfPermission(this, s) != PackageManager.PERMISSION_GRANTED)
            ActivityCompat.requestPermissions(this, new String[]{s}, REQUEST_MICROPHONE);
    }
    private void askPermissions(){
        aksPermission(Manifest.permission.RECORD_AUDIO);
        aksPermission(Manifest.permission.BLUETOOTH);
        aksPermission(Manifest.permission.BLUETOOTH_ADMIN);
        aksPermission(Manifest.permission.ACCESS_FINE_LOCATION);
        aksPermission(Manifest.permission.READ_EXTERNAL_STORAGE);
    }
    
    @RequiresApi(api = Build.VERSION_CODES.M)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setRequestedOrientation (ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        setContentView(R.layout.activity_main);
        askPermissions();
        setSupportActionBar(findViewById(R.id.toolbar));
        initGUI();
    }

    @Override
    public void onSaveInstanceState(Bundle savedInstanceState) {
        super.onSaveInstanceState(savedInstanceState);
        Log.d("SAVE", "queue size: " + sender.addressesQueue.size());
        savedInstanceState.putStringArrayList("addressesQueue", sender.addressesQueue);
        savedInstanceState.putBoolean("isConnectionOk", sender.isConnectionOk);
    }

    @RequiresApi(api = Build.VERSION_CODES.M)
    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState){
        super.onRestoreInstanceState(savedInstanceState);
        if(sender != null) {
            try {
                sender.delete();
            } catch (Throwable throwable) {
                throwable.printStackTrace();
        }}
        sender = new BluetoothLEServiceSender(this);
        Keyboard.sender = sender;
        sender.addressesQueue = savedInstanceState.getStringArrayList("addressesQueue");
        Log.d("SAVE RESTORE", "queue size: " + sender.addressesQueue.size());
        sender.isConnectionOk = savedInstanceState.getBoolean("isConnectionOk");
        sender.nextStep();
    }

    @RequiresApi(api = Build.VERSION_CODES.M)
    private void resetMembers(){
        try {
            if (scanner != null)
                scanner.delete();
            if(connecter != null)
                connecter.delete();
            if(sender != null)
                sender.delete();
            // TODO что будет, если после удаления объекта проснётся его поток?
            if(deviceListDialogfragment != null)
                deviceListDialogfragment.dismiss();
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
        sender = new BluetoothLEServiceSender(this);
        Keyboard.sender = sender;
        deviceListDialogfragment = new DeviceListDialog(getSupportFragmentManager(), sender);
        connecter = new BluetoothLEServiceConnecter(this, deviceListDialogfragment);
        scanner = new BluetoothLEServiceScanner(this);
    }

    @RequiresApi(api = Build.VERSION_CODES.M)
    private void initGUI(){
        final Button buttonSearchDevice = findViewById(R.id.connectButton);
        buttonSearchDevice.setOnClickListener(v -> {
            try {
                if (scanner != null)
                    scanner.delete();
                if(connecter != null)
                    connecter.delete();
                if(sender != null)
                    sender.delete();
                // TODO что будет, если после удаления объекта проснётся его поток?
                if(deviceListDialogfragment != null)
                    deviceListDialogfragment.dismiss();
            } catch (Throwable throwable) {
                throwable.printStackTrace();
            }
            sender = new BluetoothLEServiceSender(this);
            Keyboard.sender = sender;
            deviceListDialogfragment = new DeviceListDialog(getSupportFragmentManager(), sender);
            connecter = new BluetoothLEServiceConnecter(this, deviceListDialogfragment);
            scanner = new BluetoothLEServiceScanner(this);
            deviceListDialogfragment.show();
            scanner.scanLeDevice(deviceListDialogfragment, connecter);
        });
        final Button buttonOpenFile = findViewById(R.id.openFile);
        buttonOpenFile.setText(R.string.openFile_button_text_open);
        buttonOpenFile.setOnClickListener(v -> {
            if (sender == null){
                Toast.makeText(this, "Urządzeń nie znaleziono", Toast.LENGTH_SHORT).show();
                return;
            }
            if(buttonOpenFile.getText() == getResources().getString(R.string.openFile_button_text_open)){
                sender.startPlaying(BluetoothLEServiceSender.Mode.file);
            } else{
                this.sender.stopPlaying();
                sender.mute();
        }});
        final Button buttonMicrophone = findViewById(R.id.microphone);
        buttonMicrophone.setText("Mikrofon");
        buttonMicrophone.setOnClickListener(v -> {
            try {
                if(deviceListDialogfragment != null){
                    deviceListDialogfragment.dismiss();
                    deviceListDialogfragment = null;
                }
                if(openDialogfragment != null){
                    openDialogfragment.dismiss();
                    openDialogfragment = null;
                }
            }catch(Throwable t){
                t.printStackTrace();
            }
            if(buttonMicrophone.getText() == "Mikrofon"){
                if(sender == null){
                    return;
                }
                sender.startPlaying(BluetoothLEServiceSender.Mode.microphone);
            }else{
                sender.stopPlaying();
                sender.mute();
            }
        });
        Keyboard.initGUI(this, sender, toneGenerator);

    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}