package com.example.kamizelka;

import android.bluetooth.BluetoothGattCharacteristic;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.os.Build;
import android.os.Handler;
import android.os.Looper;
import android.os.SystemClock;
import android.util.Log;
import android.widget.Button;

import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;

import org.jtransforms.fft.FloatFFT_1D;

import java.util.Arrays;

public class SenderFunctionalMicrophone extends SenderFunctional{
	protected final static String TAG = "BLUETOOTH SENDER_MIC";
	private final int sampleRate = 6450;
	private AudioRecord audioRecord;
	private int bufferMinSize;
	private FloatFFT_1D fft;
	private PWMArray pwmArray;
	private boolean fttRun = false;
	Button buttonMicrophone;

	private class PWMArray{
		private byte bytePWM[] = new byte[16];
		public synchronized void setBytePWM(int index, byte value){
			this.bytePWM[index] = value;
		}
		public synchronized void setBytePWM(byte[] value){
			this.bytePWM = value;
		}
		public synchronized byte[] getBytePWM(){
			Log.d(TAG, Arrays.toString(this.bytePWM));
			return this.bytePWM;
		}
	}

	@RequiresApi(api = Build.VERSION_CODES.M)
	public SenderFunctionalMicrophone(AppCompatActivity active, BluetoothLEServiceSender sender) {
		super(active, sender);
		this.pwmArray = new PWMArray();
		bufferMinSize = AudioRecord.getMinBufferSize(sampleRate, AudioFormat.CHANNEL_IN_MONO, AudioFormat.ENCODING_PCM_FLOAT);
		audioRecord = new AudioRecord(MediaRecorder.AudioSource.MIC, sampleRate, AudioFormat.CHANNEL_IN_MONO, AudioFormat.ENCODING_PCM_FLOAT, bufferMinSize);
		fft = new FloatFFT_1D(bufferMinSize);
//		buffer = new float[bufferMinSize];
		buttonMicrophone = active.findViewById(R.id.microphone);
	}

	@Override
	@RequiresApi(api = Build.VERSION_CODES.M)
	public void startPlaying(){
		isPlaying = true;
		audioRecord.startRecording();
		fttRun = true;
		sender.nextStep = sender.addressesQueue.size();
		buttonMicrophone.setText(R.string.openFile_buttonMic_text_stop);
		new Thread(new Runnable(){
			@Override
			public void run() {
				SenderFunctionalMicrophone.this.fftMic();
			}
		}).start();
		nextStep();
	}

	@Override
	public void stopPlaying(){
		if (audioRecord != null) audioRecord.stop();
		buttonMicrophone.setText("Mikrofon");
		isPlaying = false;
		fttRun = false;
		// TODO посылать всем сигнал об остановке.
	}

	@RequiresApi(api = Build.VERSION_CODES.M)
	@Override
	public void nextStep(){
		try {
//		Log.d(TAG, " NextStep Mic");
			if (sender.nextStep >= sender.addressesQueue.size()) {
				sender.nextStep = 0;
				if (isPlaying) {
//				Log.d(TAG, " is playing");
					if (SystemClock.uptimeMillis() - time < Defines.SEND_FILE_PERIOD_MS) {
						for (int i = 0; i < sender.gatts.size(); ++i) {                                                // Если какое - то устройство ещё не готово, значит не выходим из функции.
							if (!sender.gatts.get(i).getState().equals(GattHolder.GattHolderState.ready)) {
								nextStepLock = true;
								sender.nextStep = i;
								nextStep();
								return;
							}
						}
//					Log.d(TAG, "time = " + (Defines.SEND_MIC_PERIOD_MS - (SystemClock.uptimeMillis() - time)));
						final Handler handler = new Handler(Looper.getMainLooper());
						handler.postDelayed(new Runnable() {
							@Override
							public void run() {
								nextStepLock = false;
								//sender.nextStep = sender.addressesQueue.size();
								nextStep();
							}
						}, Defines.SEND_MIC_PERIOD_MS - (SystemClock.uptimeMillis() - time));
						return;
					}
				}
			}
//		Log.d(TAG, " NextStepMic, next step: "+ sender.nextStep + " addressesQueue.size: " + sender.addressesQueue.size());
			if (sender.nextStep >= sender.gatts.size()) {
				sender.makeNewConnection();
			} else {
				// TODO Неуловимая ошибка.
//			Log.d(TAG, " state: " + gatts.get(nextStep).getState());
				switch (sender.gatts.get(sender.nextStep).getState()) {
					case notReady:                                                                                    // Если устройство не соеденено.
						sender.makeNewConnection();
						break;
					case connected:                                                                                    // Если у устройством есть соединение.
						Log.d(TAG, "trying to write cher, next step : " + sender.nextStep);
						sender.gatts.get(sender.nextStep).playerCharacteristic.setValue(sender.bluetoothAdapter.getAddress());
						sender.gatts.get(sender.nextStep).gatt.writeCharacteristic(sender.gatts.get(sender.nextStep).playerCharacteristic);        // Записываем на него свой аддресс.
						break;
					case written:                                                                                    // Если аддресс уже записан.
						sender.gatts.get(sender.nextStep).gatt.readCharacteristic(sender.gatts.get(sender.nextStep).readyCharacteristic);        // Опрашиваем удалённое устройство, готово ли оно.
						break;
					case ready:                                                                                        // Если устройство нашло свой левыц канал и готово.
						if (!isPlaying) {                                                                            // Если пользователь ещё не открыл файл для проигрывания.
							for (GattHolder g : sender.gatts) {                                                        // Если какое - то устройство ещё не готово, значит не выходим из функции.
								if (!g.getState().equals(GattHolder.GattHolderState.ready)) {
									++sender.nextStep;
									nextStep();
									return;
								}
							}
						} else {                                                                            // Если уже проигрываем музыку.
							sender.gatts.get(sender.nextStep).dataCharacteristic.setValue(this.pwmArray.getBytePWM());    // Передаём данные.
							try {
								sender.gatts.get(sender.nextStep).dataCharacteristic.setWriteType(BluetoothGattCharacteristic.WRITE_TYPE_NO_RESPONSE);
							} catch (Exception e) {
								Log.w(TAG, "ERROR: " + e.toString());
								sender.nextStep = 0;
							}
							sender.gatts.get(sender.nextStep).gatt.writeCharacteristic(sender.gatts.get(sender.nextStep).dataCharacteristic);
						}
						break;
					case disable:
				}
			}
			if (nextStepLock) {
				sender.nextStep = sender.addressesQueue.size();
				nextStepLock = false;
			} else {
				if (sender.nextStep == 0)
					time = SystemClock.uptimeMillis();
				++sender.nextStep;
			}
		}catch(Exception e){
			Log.wtf(TAG, "ERROR!!!!!!!!!!!");
			sender.nextStep = 0;
		}
	}

	@RequiresApi(api = Build.VERSION_CODES.M)
	void fftMic(){
		while(fttRun) {
			float[] buffer;
			buffer = new float[bufferMinSize];
			audioRecord.read(buffer, 0, bufferMinSize, AudioRecord.READ_BLOCKING);
			fft.realForward(buffer);
			float localMax = Float.MIN_VALUE;
			int maxValueFreq = -1;
			float[] result = new float[buffer.length / 2];
			for (int s = 0; s < result.length; ++s) {
				float re = buffer[s * 2];
				float im = buffer[s * 2 + 1];
				result[s] = (float) Math.sqrt(re * re + im * im) / result.length;
				if (result[s] > localMax) {
					maxValueFreq = s;
					localMax = result[s];
				}
			}
			localMax *= 1000;
//			Log.i(TAG, "Sent bytes. Freq: " + maxValueFreq + " value: " + localMax + "\n");

//			maxValueFreq -= 50;
//			maxValueFreq = maxValueFreq / 41;

			if (maxValueFreq > 300) {
				maxValueFreq = 0;
			} else if (maxValueFreq > 250) {
				maxValueFreq = 1;
			} else if (maxValueFreq > 200) {
				maxValueFreq = 2;
			} else if (maxValueFreq > 175) {
				maxValueFreq = 3;
			} else if (maxValueFreq > 150) {
				maxValueFreq = 4;
			} else if (maxValueFreq > 125) {
				maxValueFreq = 5;
			} else if (maxValueFreq > 100) {
				maxValueFreq = 6;
			} else if (maxValueFreq > 80) {
				maxValueFreq = 7;
			} else if (maxValueFreq > 60) {
				maxValueFreq = 11;
			} else if (maxValueFreq > 40) {
				maxValueFreq = 12;
			} else if (maxValueFreq > 30) {
				maxValueFreq = 13;
			} else if (maxValueFreq > 20) {
				maxValueFreq = 14;
			} else if (maxValueFreq > 10) {
				maxValueFreq = 15;
			} else {
				maxValueFreq = 0;
			}

			byte[] b = new byte[16];
			for (int iii = 0; iii < 16; ++iii) {
				b[iii] = (byte) 0x00;
			}
			b[maxValueFreq] = (byte) ((int) localMax);
			Log.d(TAG, "localMax: " + localMax + ", byte: " + b[maxValueFreq] + ", maxValueFreq: " + maxValueFreq);
			this.pwmArray.setBytePWM(b);
			int ii = 1;
			for(int i = 1; i < 10000; ++i){
				ii = i * ii;			// TODO Глупая задержка!!!
			}
//			Log.i(TAG, "ii: " + ii);
		}
	}
}
