/*
 * main.h
 *
 *  Created on: 14 kwi 2021
 *      Author: Kashey
 */

#ifndef MAIN_MAIN_H_
#define MAIN_MAIN_H_

#include <stdint.h>
#include <string.h>
#include <stdbool.h>
#include <stdio.h>
#include "nvs.h"
#include "nvs_flash.h"

#include "esp_bt.h"
#include "esp_gap_ble_api.h"
#include "esp_gattc_api.h"
#include "esp_gatt_defs.h"
#include "esp_bt_main.h"
#include "esp_gatt_common_api.h"
#include "esp_log.h"
#include "esp_bt_device.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "pwm.h"

#define MTU_VALUE 32

#define PROFILE_NUM	  		1						// Количество профилей.
#define PROFILE_A_APP_ID 	0						// Номер профиля.

#define GATTC_TAG "GATTC_DEMO"
#define DATA_SERVICE_UUID			{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xf7, 0x3d, 0x80, 0x07}			// UUID нужного сервиса.
#define INFO_SERVICE_UUID   		{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd0, 0x6d, 0x47, 0x58}					// UUID сервиса.
#define LEFT_CHANNEL_CHAR_UUID		{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xf5, 0x3d, 0x80, 0x07}			// UUID нужной службы.
#define TRANSFER_CHAR_UUID			{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xfb, 0x3d, 0x80, 0x07}			// UUID нужной службы.
#define DATA_CHAR_UUID				{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xf8, 0x3d, 0x80, 0x07}			// UUID нужной службы.
#define SERIAL_NUMBER_CHAR_UUID		{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd1, 0x6d, 0x47, 0x58}
static uint8_t serialNumber[] = {'0', '0', '0', '0', '0', '0', '0', '0', '0', '1'};

/* Declare static functions */
static void esp_gap_cb(esp_gap_ble_cb_event_t event, esp_ble_gap_cb_param_t *param);
static void esp_gattc_cb(esp_gattc_cb_event_t event, esp_gatt_if_t gattc_if, esp_ble_gattc_cb_param_t *param);
static void gattc_profile_event_handler(esp_gattc_cb_event_t event, esp_gatt_if_t gattc_if, esp_ble_gattc_cb_param_t *param);

typedef struct GATTsServiseHandler{
    uint16_t start_handle;          /*!< Service start handle */
    uint16_t end_handle;            /*!< Service end handle */
    esp_gatt_id_t srvc_id;          /*!< Service id, include service uuid and other information */
}GATTsServiseHandler;

struct gattc_profile_inst {
	esp_gattc_cb_t gattc_cb;								// функции обработки события GATT
	uint16_t gattc_if;										// Интерфейс клиента GATT
	uint16_t app_id;										// ID приложения
	uint16_t conn_id;										// ID соединения
	uint16_t data_service_start_handle;						// Service start handle
	uint16_t data_service_end_handle;						// Service end handle
	uint16_t info_service_start_handle;						// Service start handle
	uint16_t info_service_end_handle;						// Service end handle
	uint16_t serial_number_char_handle;						// Char handle
	uint16_t left_channel_char_handle;						// Char handle
	uint16_t transfer_char_handle;							// Char handle
	uint16_t data_channel_char_handle;						// Char handle
	uint16_t char_handle;
	esp_bd_addr_t remote_bda;								// Адресс удалённого устройства (сервера), подключённого к этому клиенту.
};







#endif /* MAIN_MAIN_H_ */
