/*
 * main.c
 *
 *  Created on: 11 kwi 2021
 *      Author: Kashey
 */
#include "main.h"
#include "profiles.h"
#include "pwm.h"

static void gap_event_handler(esp_gap_ble_cb_event_t event, esp_ble_gap_cb_param_t *param){
	switch (event) {
	case ESP_GAP_BLE_ADV_DATA_SET_COMPLETE_EVT:															// Событие завершения установки параметров рассылки широковещательных пакетов.
		adv_config_done &= (~adv_config_flag);
		if (adv_config_done == 0)
			esp_ble_gap_start_advertising(&adv_params);													// Рассылка рекламных широковещательных пакетов.
		break;
	case ESP_GAP_BLE_SCAN_RSP_DATA_SET_COMPLETE_EVT:													// Событие завершения установки параметров ответов на рекламу.
		adv_config_done &= (~scan_rsp_config_flag);
		if (adv_config_done == 0)
			esp_ble_gap_start_advertising(&adv_params);													// Рассылка рекламных широковещательных пакетов.
		break;
	case ESP_GAP_BLE_ADV_START_COMPLETE_EVT:															// Событие завершения запуска отсылки рекламных пакетов. Нужна для проверки успешно ли пошла реклама.
		if (param->adv_start_cmpl.status != ESP_BT_STATUS_SUCCESS)
			ESP_LOGE(GATTS_TAG, "Advertising start failed\n");
		break;
	case ESP_GAP_BLE_ADV_STOP_COMPLETE_EVT:																// Событие завершения отсылки рекламы.
		if (param->adv_stop_cmpl.status != ESP_BT_STATUS_SUCCESS)
			ESP_LOGE(GATTS_TAG, "Advertising stop failed\n");
		else
			ESP_LOGI(GATTS_TAG, "Stop adv successfully\n");
		break;
	case ESP_GAP_BLE_UPDATE_CONN_PARAMS_EVT:															// Событие, запускаемое обновлением параметров соединения.
		 ESP_LOGI(GATTS_TAG, "update connection params status = %d, min_int = %d, max_int = %d,conn_int = %d,latency = %d, timeout = %d",
				  param->update_conn_params.status, param->update_conn_params.min_int, param->update_conn_params.max_int,
				  param->update_conn_params.conn_int, param->update_conn_params.latency, param->update_conn_params.timeout);
		break;
	default:
		break;
}}

// Функция содержит логику записи длинной харрактеристики.
void example_write_event_env(esp_gatt_if_t gatts_if, prepare_type_env_t *prepare_write_env, esp_ble_gatts_cb_param_t *param){
	ESP_LOGI(GATTS_TAG, "Write is prep!");
	esp_gatt_status_t status = ESP_GATT_OK;
	ESP_LOGE(GATTS_TAG, "Write operation! connection id: %i, transfer id: %i, attribute handle: %i, offset: %i, need_rsp: %i, is_prep: %i, data len: %i\n",
			param->write.conn_id, param->write.trans_id, param->write.handle, param->write.offset, param->write.need_rsp, param->write.is_prep, param->write.len);
	for(int i = 0; i < param->write.len; ++i){
		printf("%X ", param->write.value[i]);
	}
	printf("\n");
	if (param->write.need_rsp){																		// Если клиент отправляет запрос записи с подтверждением (Write Request or a Prepare Write Request), сервер должен ответить.
																											// Но если клиент отправляет запрос записи без подтверждения (Write Without Response command) отвечать не надо.
		if (param->write.is_prep){																			// Запрашивает ли клиент длинную харрактеристику записи (больше 21 бита).
			if (prepare_write_env->prepare_buf == NULL) {													// Если харрактеристика длинная, то подготавливается нескольких ответов на запись.
				prepare_write_env->prepare_buf = (uint8_t *)malloc(PREPARE_BUF_MAX_SIZE*sizeof(uint8_t));	// Выделяем место для длинного ответа.
				prepare_write_env->prepare_len = 0;
				if (prepare_write_env->prepare_buf == NULL) {												// Если нет памяти для буфера ответа, сообщаем об ошибке.
					ESP_LOGE(GATTS_TAG, "Gatt_server prep no mem\n");
					status = ESP_GATT_NO_RESOURCES;
				}
			} else {
				if(param->write.offset > PREPARE_BUF_MAX_SIZE)												// Если буфер создан успешно, проверяем, подходит ли длина и смещение для буфера.
					status = ESP_GATT_INVALID_OFFSET;
				else if ((param->write.offset + param->write.len) > PREPARE_BUF_MAX_SIZE)
					status = ESP_GATT_INVALID_ATTR_LEN;
			}
			esp_gatt_rsp_t *gatt_rsp = (esp_gatt_rsp_t *)malloc(sizeof(esp_gatt_rsp_t));					// Создаём структуру ответа клиенту.
			gatt_rsp->attr_value.len = param->write.len;													// Подготавливаем ответ.
			gatt_rsp->attr_value.handle = param->write.handle;												// Он создаётся с теми же параметрами как у запроса на запись:
			gatt_rsp->attr_value.offset = param->write.offset;												// длина, указатель атрибута и смещение.
			gatt_rsp->attr_value.auth_req = ESP_GATT_AUTH_REQ_NONE;											// Тип аунтентификации GATT, необходимый для записи в эту харрактеристику: без аутентификации.
			memcpy(gatt_rsp->attr_value.value, param->write.value, param->write.len);
			// Отправляем ответ.
			esp_err_t response_err = esp_ble_gatts_send_response(gatts_if, param->write.conn_id, param->write.trans_id, status, gatt_rsp);
			if (response_err != ESP_OK)
			   ESP_LOGE(GATTS_TAG, "Send response error\n");
			// Освобождаем память.
			free(gatt_rsp);
			if (status != ESP_GATT_OK)
				return;
			memcpy(prepare_write_env->prepare_buf + param->write.offset, param->write.value, param->write.len);	// Входящие данные копируются в созданый буфер.
			prepare_write_env->prepare_len += param->write.len;													// Длина буфера увеличивается на смещение.
			prepare_write_env->handle = param->write.handle;
		}else{
			esp_ble_gatts_send_response(gatts_if, param->write.conn_id, param->write.trans_id, status, NULL);	// Отправка ответа на запись.
}}}

// Функция обработки исполнительного запроса на запись.
// Исполнительная запись используется для подтверждения или отмены процедуры записи,
// выполненной ранее процедурой записи длинных характеристик.
void example_exec_write_event_env(prepare_type_env_t *prepare_write_env, esp_ble_gatts_cb_param_t *param){
	// Проверяем флаг exec_write_flag в параметрах полученных с событием.
	if (param->exec_write.exec_write_flag == ESP_GATT_PREP_WRITE_EXEC)
		esp_log_buffer_hex(GATTS_TAG, prepare_write_env->prepare_buf, prepare_write_env->prepare_len);		// Подтверждение записи.
	else
		ESP_LOGI(GATTS_TAG,"ESP_GATT_PREP_WRITE_CANCEL");													// Отмена записи, удаляем данные.
	if (prepare_write_env->prepare_buf) {
		gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value =
				realloc(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value, prepare_write_env->prepare_len * sizeof(uint8_t));
		memcpy(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value, prepare_write_env->prepare_buf, prepare_write_env->prepare_len);
		gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_len = prepare_write_env->prepare_len;
		free(prepare_write_env->prepare_buf);																// Удаляем буфер, созданный для записи длинных сообщений.
		prepare_write_env->prepare_buf = NULL;																// Устанавливаем указатель в NULL для последующего использования.
	}
	prepare_write_env->prepare_len = 0;

	ESP_LOGI(GATTS_TAG,"Write data to: %u", param->write.handle);

	esp_ble_gatts_send_indicate(gl_profile_tab[PROFILE_A_APP_ID].gatts_if, conn_id_left_channel, prepare_write_env->handle,
			gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_len, gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value, false);



	//esp_ble_gatts_send_indicate(gl_profile_tab[PROFILE_A_APP_ID].gatts_if, conn_id_left_channel, prepare_write_env->handle, 10, uVal, false);










}
// Функция ответа на чтение клиентом. Эта функция необходима, если байт автоответа установлен в NULL при создании характеристики или дескриптора.
static void sendResp(uint8_t valuesSize, uint8_t* values, esp_gatt_if_t gatts_if, esp_ble_gatts_cb_param_t *param){
	esp_gatt_rsp_t rsp;																						// Создание ответа.
	memset(&rsp, 0, sizeof(esp_gatt_rsp_t));
	rsp.attr_value.handle = param->read.handle;																// Указатель атрибута.
	rsp.attr_value.len = valuesSize;																		// Количество передаваемых значений.
	for(int i = 0; i < valuesSize; ++i){
		printf(" %x, ", values[i]);
		rsp.attr_value.value[i] = values[i];
	}
	printf("\n");
	esp_ble_gatts_send_response(gatts_if, param->read.conn_id, param->read.trans_id, ESP_GATT_OK, &rsp);	// Ответ.
}
// Обработчик событий GATT сервера.
static void gatts_profile_a_event_handler(esp_gatts_cb_event_t event, esp_gatt_if_t gatts_if, esp_ble_gatts_cb_param_t *param) {
	switch (event) {
	case ESP_GATTS_REG_EVT:																					// Событие регистрации профиля.
		profilesInit(ESP_GATTS_REG_EVT, param, gatts_if);
		break;
	case ESP_GATTS_CREATE_EVT:																				// Событие возникающее после создания сервиса. Используется для добавления в него харрактеристик.
		profilesInit(ESP_GATTS_CREATE_EVT, param, gatts_if);
		break;
	case ESP_GATTS_ADD_CHAR_EVT: 																			// Событие при завершении добавления харрактеристики. Используется для добавления дескриптора.
		profilesInit(ESP_GATTS_ADD_CHAR_EVT, param, gatts_if);
		break;
	case ESP_GATTS_ADD_CHAR_DESCR_EVT:																		// Событие запускается после добавления дескриптора харрактеристики.
		profilesInit(ESP_GATTS_ADD_CHAR_DESCR_EVT, param, gatts_if);
		break;
	case ESP_GATTS_READ_EVT: 																				// Событие операции чтения клиентом.
		ESP_LOGI(GATTS_TAG, "GATT_READ_EVT, conn_id %d, trans_id %d, handle %d\n", param->read.conn_id, param->read.trans_id, param->read.handle);
		if(param->read.handle ==       gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[SERIAL_NUMBER_CHARACTERISTIC].descrs[0].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 1");
			sendResp(sizeof(vSerialNumberD), vSerialNumberD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[SERIAL_NUMBER_CHARACTERISTIC].char_handle){
			ESP_LOGI(GATTS_TAG, "Read: 2");
			sendResp(sizeof(vSerialNumberC), vSerialNumberC, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[CHANNEL_CHARACTERISTIC].descrs[0].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 3");
			sendResp(sizeof(vChannelD), vChannelD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[CHANNEL_CHARACTERISTIC].char_handle){
			ESP_LOGI(GATTS_TAG, "Read: 4");
			sendResp(sizeof(vChannelC), vChannelC, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[AUTHOR_CHARACTERISTIC].descrs[0].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 5");
			sendResp(sizeof(vAuthorD), vAuthorD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[AUTHOR_CHARACTERISTIC].char_handle){
			ESP_LOGI(GATTS_TAG, "Read: 6");
			sendResp(sizeof(vAuthorC), vAuthorC, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[TECHNICAL_IMPLEMENTATION].descrs[0].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 7");
			sendResp(sizeof(vTechnicalImplementationD), vTechnicalImplementationD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[TECHNICAL_IMPLEMENTATION].char_handle){
			ESP_LOGI(GATTS_TAG, "Read: 8");
			sendResp(sizeof(vTechnicalImplementationC), vTechnicalImplementationC, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[VERSION_CHARACTERISTIC].descrs[0].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 9");
			sendResp(sizeof(vVersionD), vVersionD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[VERSION_CHARACTERISTIC].char_handle){
			ESP_LOGI(GATTS_TAG, "Read: 10");
			sendResp(sizeof(vVersionC), vVersionC, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[DEVICE_NAME_CHARACTERISTIC].descrs[0].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 11");
			sendResp(sizeof(vDeviceNameD), vDeviceNameD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[DEVICE_NAME_CHARACTERISTIC].char_handle){
			ESP_LOGI(GATTS_TAG, "Read: 12");
			sendResp(sizeof(vDeviceNameC), vDeviceNameC, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].descrs[0].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 13");
			sendResp(sizeof(vDataD), vDataD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].descrs[1].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 14");
			sendResp(sizeof(vDataDescriptionD), vDataDescriptionD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[TRANSFER_DEVICE_ADDRESS_CHARACTERISTIC].descrs[1].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 14");
			sendResp(sizeof(vTransferD), vTransferD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[TRANSFER_DEVICE_ADDRESS_CHARACTERISTIC].char_handle){
			ESP_LOGI(GATTS_TAG, "Read: 14");
			sendResp(sizeof(vTransferC), vTransferC, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].descrs[1].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 14");
			sendResp(sizeof(vPlayerD), vPlayerD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].char_handle){
			ESP_LOGI(GATTS_TAG, "Read: 14");
			sendResp(sizeof(vPlayerC), vPlayerC, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].descrs[1].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 14");
			sendResp(sizeof(vLeftChannelD), vLeftChannelD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].char_handle){
			ESP_LOGI(GATTS_TAG, "Read: 14");
			sendResp(sizeof(vLeftChannelC), vLeftChannelC, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[READY_CHARACTERISTIC].descrs[1].descr_handle){
			ESP_LOGI(GATTS_TAG, "Read: 14");
			sendResp(sizeof(vReadyD), vReadyD, gatts_if, param);
		}else if(param->read.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[READY_CHARACTERISTIC].char_handle){
			ESP_LOGI(GATTS_TAG, "Read: 14");
			sendResp(sizeof(vReadyC), vReadyC, gatts_if, param);
		}else{
			ESP_LOGI(GATTS_TAG, "Read: 15");
			uint8_t data[] = {(charValue & 0x000000FF), (charValue & 0x0000FF00) >> 8, (charValue & 0x00FF0000) >> 16, (charValue & 0xFF000000) >> 24};
			sendResp(sizeof(data), data, gatts_if, param);
		}
		break;
	case ESP_GATTS_WRITE_EVT: {																				// Событие операции записи клиентом.
		transmissionTimer = esp_timer_get_time();
	    ESP_LOGI(TAG, "Write time: %lld ", transmissionTimer);
		ESP_LOGI(GATTS_TAG, "GATT_WRITE_EVT, conn_id %d, trans_id %d, handle %d", param->write.conn_id, param->write.trans_id, param->write.handle);
		if (!param->write.is_prep){																			// Если запрашивается короткая запись (до 23 байт).
			ESP_LOGI(GATTS_TAG, "Write is no prep!");
			if(param->write.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].char_handle){
//				audioBuffer = *((audio_buffer_t*)realloc(&audioBuffer, (audioBuffer.audioDataBufferLen + 1) * sizeof(audioBuffer)));
//				audioBuffer.audioDataBuffer[audioBuffer.audioDataBufferLen] = realloc(audioBuffer.audioDataBuffer[audioBuffer.audioDataBufferLen], param->write.len * sizeof(uint8_t));
				printf("mcmcopy, len = %d, sizeof: %d", param->write.len, sizeof(param->write.value));
//				memcpy(audioBuffer.audioDataBuffer[audioBuffer.audioDataBufferLen], param->write.value, param->write.len);
// TODO
				gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value =
						realloc(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value, param->write.len * sizeof(uint8_t));
				memcpy(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value, param->write.value, param->write.len);
				gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_len = param->write.len;

				esp_err_t err = esp_ble_gatts_send_indicate(gl_profile_tab[PROFILE_A_APP_ID].gatts_if, conn_id_left_channel, param->write.handle,
					gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_len, gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value, false);

				runPWM(param->write.value);

//				++audioBuffer.audioDataBufferLen;

//				gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_len = param->write.len;
//				gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value = param->write.value;
				ESP_LOGI(GATTS_TAG, "GATT_WRITE_EVT, value len %d, value: ", param->write.len);
				esp_log_buffer_hex(GATTS_TAG, param->write.value, param->write.len);
				ESP_LOGI(GATTS_TAG, ", value in char: ");
				esp_log_buffer_hex(GATTS_TAG, gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[SERIAL_NUMBER_CHARACTERISTIC].value->attr_value, gl_profile_tab[PROFILE_A_APP_ID].servs[INFO_SERVICE].chars[SERIAL_NUMBER_CHARACTERISTIC].value->attr_len);
				ESP_LOGI(GATTS_TAG, "\n *************************** \n");
				break;
			}else if(param->write.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].char_handle){	// Если подключился проигрыватель.
				ESP_LOGI(GATTS_TAG, "\n Player connected \n");
				conn_id_player = param->write.conn_id;																										// Записываем id соединения.
				gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_len = param->write.len;		// Записываем его аддресс.
				memcpy(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value, param->write.value, param->write.len * sizeof(uint8_t));
				if(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_len == sizeof(vLeftChannelC) &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[0] == 0 &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[1] == 0 &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[2] == 0 &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[3] == 0 &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[4] == 0 &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[5] == 0){	// Если вторая часть безрукавки не была подключена.
					ESP_LOGI(GATTS_TAG, "\n Not ready, lack of left device. \n");
					esp_ble_gap_start_advertising(&adv_params);																								// Продолжаем рассылку широковещательных пакетов.
				}else{
					ESP_LOGI(GATTS_TAG, "\n change to ready \n");
					*(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[READY_CHARACTERISTIC].value->attr_value) = (uint8_t){0x01};
				}
				//break;
			}else if(param->write.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].char_handle){	// Если подключилась вторая часть безрукавки.
				ESP_LOGI(GATTS_TAG, "\n LEFT DEVICE CONNECTED \n");
				conn_id_left_channel = param->write.conn_id;																								// Записываем id соединения, куда будем отправлять увадомления.
				gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_len = param->write.len;	// Записываем значение адресса.
				memcpy(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value, param->write.value, param->write.len * sizeof(uint8_t));
				if(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_len == sizeof(vPlayerC) &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[0] == 0 &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[1] == 0 &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[2] == 0 &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[3] == 0 &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[4] == 0 &&
						gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value[5] == 0){	// Если проигрыватель не был подключен.
					ESP_LOGI(GATTS_TAG, "\n Not Ready, lack of player. \n");
					esp_ble_gap_start_advertising(&adv_params);																								// Продолжаем рассылку широковещательных пакетов.
				}else{
					ESP_LOGI(GATTS_TAG, "\n change to ready \n");
					*(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[READY_CHARACTERISTIC].value->attr_value) = (uint8_t){0x01};
				}

				if(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[TRANSFER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_len != sizeof(vTransferC)){
					*(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[READY_CHARACTERISTIC].value->attr_value) = (uint8_t){0x01};
				}
				//break;
			}else if(param->write.handle == gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[TRANSFER_DEVICE_ADDRESS_CHARACTERISTIC].char_handle){
				gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[TRANSFER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_len = param->write.len;
				memcpy(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[TRANSFER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value, param->write.value, param->write.len * sizeof(uint8_t));
				if(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_len != sizeof(vReadyC)){
					*(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[READY_CHARACTERISTIC].value->attr_value) = (uint8_t){0x01};
				}
				//break;
			}
		}
		example_write_event_env(gatts_if, &a_prepare_write_env, param);
		break;
	}
	case ESP_GATTS_EXEC_WRITE_EVT:													// В конце, когда клиент закончил отправку сообщений длинной записи, он отправляет исполнительный запрос на запись (Executive Write Request).
		ESP_LOGI(GATTS_TAG,"***Write data to: %u", param->write.handle);
		ESP_LOGI(GATTS_TAG,"ESP_GATTS_EXEC_WRITE_EVT");															// Этот запрос запускает это событие.
		esp_ble_gatts_send_response(gatts_if, param->write.conn_id, param->write.trans_id, ESP_GATT_OK, NULL);	// Сервер отправляет ответ на исполнительный запрос на запись.
		example_exec_write_event_env(&a_prepare_write_env, param);												// Сервер обрабатывает запрос на запись.
		break;
	case ESP_GATTS_MTU_EVT:																						// Событие завершение конфигурации MTU.
		ESP_LOGI(GATTS_TAG, "ESP_GATTS_MTU_EVT, MTU %d", param->mtu.mtu);
		break;
	case ESP_GATTS_UNREG_EVT:																					// Событие удаления регистрации ID приложения.
		break;
	case ESP_GATTS_ADD_INCL_SRVC_EVT:
		break;
	case ESP_GATTS_DELETE_EVT:
		break;
	case ESP_GATTS_START_EVT:																					// Событие запуска службы.
		ESP_LOGI(GATTS_TAG, "SERVICE_START_EVT, status %d, service_handle %d\n", param->start.status, param->start.service_handle);
		break;
	case ESP_GATTS_STOP_EVT:
		break;
		// Это событие запускается, когда клиент подключился к GATTS, используется для обновления параметров подключения,
		// таких как задержка, минимальный интервал подключения, максимальный интервал подключения и время ожидания.
		// Процедуру обновления параметров соединения необходимо выполнить только один раз,
		// поэтому обработчик события соединения профиля B не включает функцию esp_ble_gap_update_conn_params ().
	case ESP_GATTS_CONNECT_EVT: {
		esp_ble_conn_update_params_t conn_params = {0};						// Параметры подключения.
		memcpy(conn_params.bda, param->connect.remote_bda, sizeof(esp_bd_addr_t));
		/* For the IOS system, please reference the apple official documents about the ble connection parameters restrictions. */
		conn_params.latency = 0;
		conn_params.max_int = 0x20;		// max_int = 0x20*1.25ms = 40ms
		conn_params.min_int = 0x10;		// min_int = 0x10*1.25ms = 20ms
		conn_params.timeout = 400;		// timeout = 400*10ms = 4000ms
		ESP_LOGI(GATTS_TAG, "ESP_GATTS_CONNECT_EVT, conn_id %d, remote %02x:%02x:%02x:%02x:%02x:%02x:", param->connect.conn_id, param->connect.remote_bda[0], param->connect.remote_bda[1], param->connect.remote_bda[2], param->connect.remote_bda[3], param->connect.remote_bda[4], param->connect.remote_bda[5]);
		gl_profile_tab[PROFILE_A_APP_ID].conn_id = param->connect.conn_id;	// Идентификатор соединения сохраняется в таблице профиля.
		esp_ble_gap_update_conn_params(&conn_params);
		esp_ble_gap_start_advertising(&adv_params);
		break;	}
	case ESP_GATTS_DISCONNECT_EVT:											// Событие после отключения клиента.
		if(param->disconnect.conn_id == conn_id_player){
			uint8_t u[] = {0, 0, 0, 0, 0, 0};
			memcpy(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value, u, sizeof(u));
			gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_len = sizeof(vPlayerC)/sizeof(uint8_t);		// Записываем его аддресс.
			*(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[READY_CHARACTERISTIC].value->attr_value) = (uint8_t){0x00};
			ESP_LOGI(GATTS_TAG, "Player disconnected, change state to not ready");
			ESP_LOGI(GATTS_TAG, "Ready is: %d", *(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[READY_CHARACTERISTIC].value->attr_value));
		}else if(param->disconnect.conn_id == conn_id_left_channel){
			uint8_t u[] = {0, 0, 0, 0, 0, 0};
			memcpy(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_value, u, sizeof(u)/sizeof(uint8_t));
			gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC].value->attr_len = sizeof(vLeftChannelC)/sizeof(uint8_t);	// Записываем значение адресса.
			*(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[READY_CHARACTERISTIC].value->attr_value) = (uint8_t){0x00};
			ESP_LOGI(GATTS_TAG, "Left channel disconnected, change state to not ready");
			ESP_LOGI(GATTS_TAG, "Ready is: %d", *(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[READY_CHARACTERISTIC].value->attr_value));
		}
		ESP_LOGI(GATTS_TAG, "ESP_GATTS_DISCONNECT_EVT, disconnect reason 0x%x", param->disconnect.reason);
		esp_ble_gap_start_advertising(&adv_params);
		break;
	case ESP_GATTS_CONF_EVT:												// Событие получения подтверждения.
		ESP_LOGI(GATTS_TAG, "ESP_GATTS_CONF_EVT, status %d attr_handle %d", param->conf.status, param->conf.handle);
		if (param->conf.status != ESP_GATT_OK){
			esp_log_buffer_hex(GATTS_TAG, param->conf.value, param->conf.len);
		}
		break;
	case ESP_GATTS_CLOSE_EVT:
		ESP_LOGI(GATTS_TAG, "ESP_GATTS_CLOSE_EVT");
		break;
	case ESP_GATTS_OPEN_EVT:
	case ESP_GATTS_CANCEL_OPEN_EVT:
	case ESP_GATTS_LISTEN_EVT:
	case ESP_GATTS_CONGEST_EVT:
	default:
		break;
}}

// Регистрация обработчиков событий GATTS
static void gatts_event_handler(esp_gatts_cb_event_t event, esp_gatt_if_t gatts_if, esp_ble_gatts_cb_param_t *param){
	if (event == ESP_GATTS_REG_EVT) {						// Событие регистрации профиля.
		if (param->reg.status == ESP_GATT_OK) {
			// Когда профиль будет зарегистрирован, каждому профилю будет назначен свой GATT интерфейс gatt_if.
			gl_profile_tab[param->reg.app_id].gatts_if = gatts_if;
		} else {
			ESP_LOGI(GATTS_TAG, "Reg app failed, app_id %04x, status %d\n", param->reg.app_id, param->reg.status);
			return;
	}}
	do {	// Передаём событие обработчику событий профиля. Если событие не специализированно (ESP_GATT_IF_NONE), передаём его всем профилям.
		for (int idx = 0; idx < PROFILE_NUM; idx++) {
			if (gatts_if == ESP_GATT_IF_NONE || gatts_if == gl_profile_tab[idx].gatts_if) {
				if (gl_profile_tab[idx].gatts_cb) {
					gl_profile_tab[idx].gatts_cb(event, gatts_if, param);
	}}}} while (0);
}

void ATaskFunction( void *pvParameters ){
	vTaskDelay( 2000 / portTICK_RATE_MS );
	for( ;; ){
		++charValue;
//		printf("task1, charValue = %i ", charValue);
		if(gl_profile_tab[PROFILE_A_APP_ID].servs[0].chars[1].value != NULL){
			printf("char VALUE: %i ", gl_profile_tab[PROFILE_A_APP_ID].servs[0].chars[1].value->attr_len);
			for (int i = 0; i < gl_profile_tab[PROFILE_A_APP_ID].servs[0].chars[1].value->attr_len; ++i){
				printf(" %x ", gl_profile_tab[PROFILE_A_APP_ID].servs[0].chars[1].value->attr_value[i]);
			}
		}
		vTaskDelay( 5000 / portTICK_RATE_MS );
	}
	vTaskDelete( NULL );
}

void app_main(){
	registerDinamicVlaues();
	pwmInitialization();
	esp_err_t ret = nvs_flash_init();						// Инициализация библиотеки энергонезависимой памяти.
	hFlashInit(ret);
	esp_bt_controller_config_t bt_cfg = BT_CONTROLLER_INIT_CONFIG_DEFAULT();	// Создание структуры БТ контроллера.
															// Содержит свойства контроллера: размер стака, приоритет и символьную скорость.
	ret = esp_bt_controller_init(&bt_cfg);					// Инициализируем и включаем БТ контроллер.
	hCheckErr(ret, "initialize controller failed", __func__);
	ret = esp_bt_controller_enable(ESP_BT_MODE_BLE);		// Устанавливаем контроллер в BLE режим.
															// Если надо использовать оба режима,
															// BLU и BT надо использовать ESP_BT_MODE_BTDM.
															/* Есть 4 установки контроллера:
																ESP_BT_MODE_IDLE: Bluetooth выключен
																ESP_BT_MODE_BLE: BLE режим
																ESP_BT_MODE_CLASSIC_BT: BT классический режим
																ESP_BT_MODE_BTDM: Оба режима (BLE + BT Classic) */
	hCheckErr(ret, "enable controller failed", __func__);
	ret = esp_bluedroid_init();	 						// Инициализация Bluedroid stack (содержит API для BLE и BT).
	hCheckErr(ret, "init bluetooth failed", __func__);
	ret = esp_bluedroid_enable();							// Включение Bluedroid stack.
	hCheckErr(ret, "enable bluetooth failed", __func__);
	ret = esp_ble_gatts_register_callback(gatts_event_handler);	// Регистрация функций обработки события для GATTS. Назначает интерфейс GATT профилю.
	hCheckErr2(ret, "gatts register error");
	ret = esp_ble_gap_register_callback(gap_event_handler);	// Регистрация функций обработки события для GAP. Рассылка рекламы и логи.
	hCheckErr2(ret, "gap register error");
	addProfiles();
	ret = esp_ble_gatt_set_local_mtu(MTU_VALUE);					// Объявление MTU - максимальной длины ATT пакета.
	hCheckErr2(ret, "set local  MTU failed");
	ret = esp_ble_gap_set_device_name(TEST_DEVICE_NAME);	// Устанавливаем имя устройства.
	hCheckErr2(ret, "set device name failed");
	// Также можно рекламировать настроенные необработанные данные с помощью функций esp_ble_gap_config_adv_data_raw ()
	// и esp_ble_gap_config_scan_rsp_data_raw (), которые требуют создания и передачи буфера как для рекламных данных,
	// так и для данных ответа на сканирование. В этом примере необработанные данные представлены массивами
	// raw_adv_data [] и raw_scan_rsp_data [].
	ret = esp_ble_gap_config_adv_data(&adv_data);		// Конфигурация рекламных пакетов.
	hCheckErr2(ret, "config adv data failed");
	adv_config_done |= adv_config_flag;
	ret = esp_ble_gap_config_adv_data(&scan_rsp_data);	// Конфигурация ответов на сканирование.
	hCheckErr2(ret, "config scan response data failed");
	adv_config_done |= scan_rsp_config_flag;
//	startTimer();
	xTaskCreate(ATaskFunction, "TASK1", 2048, NULL, 1, NULL);
	return;
}
