/*
 * main.h
 *
 *  Created on: 11 kwi 2021
 *      Author: Kashey
 */

#ifndef MAIN_MAIN_H_
#define MAIN_MAIN_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"
#include "nvs_flash.h"
#include "esp_system.h"
#include "esp_log.h"
#include "esp_bt.h"
#include "esp_gap_ble_api.h"
#include "esp_gatts_api.h"
#include "esp_bt_defs.h"
#include "esp_bt_main.h"
#include "esp_gatt_common_api.h"
#include "sdkconfig.h"
#include "profiles.h"
#include "timer.h"

#define TEST_DEVICE_NAME "Kamizelka"			// Имя сервера. Именно по этому имени клиент узнаёт сервер и устанавливает соединение чтобы поискать нужные ему сервисы.
#define TEST_MANUFACTURER_DATA_LEN  17

#define PREPARE_BUF_MAX_SIZE 1024
#define MTU_VALUE 32

static uint8_t adv_config_done = 0;
#define adv_config_flag (1 << 0)
#define scan_rsp_config_flag (1 << 1)

static int64_t transmissionTimer;

static uint8_t adv_service_uuid128[32] = {
	/* LSB <--------------------------------------------------------------------------------> MSB */
	//first uuid, 16bit, [12],[13] is the value
	0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0xEE, 0x00, 0x00, 0x00,
	//second uuid, 32bit, [12], [13], [14], [15] is the value
	0xfb, 0x34, 0x9b, 0x5f, 0x80, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00,
};

// Настройка широковещательных сообщений. Полезная нагрузка рекламы может содержать до 31 байта данных.
//static uint8_t test_manufacturer[TEST_MANUFACTURER_DATA_LEN] =  {0x12, 0x23, 0x45, 0x56};
static esp_ble_adv_data_t adv_data = {
	.set_scan_rsp = false,												// Set this advertising data as scan response or not
	.include_name = true,												// Advertising data include device name or not
	.include_txpower = false,											// Advertising data include TX power
	.min_interval = 0x0006, 											// Минимальный период подключения ведомого устройства
	.max_interval = 0x0010, 											// Максимальный период подключения. Time = max_interval * 1.25 msec
	.appearance = 0x00,													// External appearance of device
	.manufacturer_len = 0, 												// Manufacturer data length (TEST_MANUFACTURER_DATA_LEN)
	.p_manufacturer_data =  NULL, 										// Manufacturer data point (&test_manufacturer[0])
	.service_data_len = 0,												// Service data length
	.p_service_data = NULL,												// Service data point
	.service_uuid_len = sizeof(adv_service_uuid128),					// Service uuid length
	.p_service_uuid = adv_service_uuid128,								// Service uuid array point
	.flag = (ESP_BLE_ADV_FLAG_GEN_DISC | ESP_BLE_ADV_FLAG_BREDR_NOT_SPT),// Advertising flag of discovery mode, see BLE_ADV_DATA_FLAG detail
};
// scan response data
static esp_ble_adv_data_t scan_rsp_data = {
	.set_scan_rsp = true,
	.include_name = true,
	.include_txpower = true,
	//.min_interval = 0x0006,
	//.max_interval = 0x0010,
	.appearance = 0x00,
	.manufacturer_len = 0, //TEST_MANUFACTURER_DATA_LEN,
	.p_manufacturer_data =  NULL, //&test_manufacturer[0],
	.service_data_len = 0,
	.p_service_data = NULL,
	.service_uuid_len = sizeof(adv_service_uuid128),
	.p_service_uuid = adv_service_uuid128,
	.flag = (ESP_BLE_ADV_FLAG_GEN_DISC | ESP_BLE_ADV_FLAG_BREDR_NOT_SPT),
};

// Параметры рекламных широковещательных пакетов.
static esp_ble_adv_params_t adv_params = {
	.adv_int_min		= 0x20,						// Минимальный временной интервал сканирования. *0.625s
	.adv_int_max		= 0x40,						// Максимальный временной интервал сканирования. *0.625s
	.adv_type		  	= ADV_TYPE_IND,				// Тип широковещательных пакетов.
	.own_addr_type		= BLE_ADDR_TYPE_PUBLIC,		// Владелец адресса устройства блютуз.
	//.peer_addr			=
	//.peer_addr_type	   =
	.channel_map		= ADV_CHNL_ALL,				// Карта широковещательных каналов.
	.adv_filter_policy = ADV_FILTER_ALLOW_SCAN_ANY_CON_ANY,	// Политика фильтрации.
};

// Буферная структура подготовки для обработки длинной характеристики записи.
typedef struct {
	uint16_t			handle;
	uint8_t				 *prepare_buf;
	int					 prepare_len;
} prepare_type_env_t;

static prepare_type_env_t a_prepare_write_env;

// Функция содержит логику записи длинной харрактеристики.
void example_write_event_env(esp_gatt_if_t gatts_if, prepare_type_env_t *prepare_write_env, esp_ble_gatts_cb_param_t *param);
void example_exec_write_event_env(prepare_type_env_t *prepare_write_env, esp_ble_gatts_cb_param_t *param);

typedef struct {
	uint8_t** audioDataBuffer;
	uint8_t audioDataBufferLen;
}audio_buffer_t;

static audio_buffer_t audioBuffer = {
		.audioDataBuffer = NULL,
		.audioDataBufferLen = 0
};

static uint8_t conn_id_transfer = 0xF;
static uint8_t conn_id_player = 0xF;
static uint8_t conn_id_left_channel = 0xF;

#endif /* MAIN_MAIN_H_ */
