/*
 * profiles.h
 *
 *  Created on: 7 mar 2021
 *      Author: Kashey
 */

/* Логика регистрации таблицы профилей была написана под влиянием: https://gist.github.com/heiko-r/f284d95141871e12ca0164d9070d61b4#file-ble_server-c-L494 */

#ifndef MAIN_PROFILES_H_
#define MAIN_PROFILES_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"
#include "nvs_flash.h"
#include "esp_system.h"
#include "esp_log.h"
#include "esp_bt.h"
#include "esp_gap_ble_api.h"
#include "esp_gatts_api.h"
#include "esp_bt_defs.h"
#include "esp_bt_main.h"
#include "esp_gatt_common_api.h"
#include "sdkconfig.h"
#include "profiles.h"

#define GATTS_TAG "GATTS_DEMO"

#define PROFILE_NUM 1																								// Количество профилей.
#define PROFILE_A_APP_ID 0																							// Номер первого профиля.

#define INFO_SERVICE_UUID   									{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd0, 0x6d, 0x47, 0x58}					// UUID сервиса.
#define SERIAL_NUMBER_CHARACTERISTIC_UUID						{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd1, 0x6d, 0x47, 0x58}
#define SERIAL_NUMBER_DESCRIPTOR_UUID							{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd2, 0x6d, 0x47, 0x58}
#define CHANNEL_CHARACTERISTIC_UUID								{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd3, 0x6d, 0x47, 0x58}
#define CHANNEL_DESCRIPTOR_UUID									{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd4, 0x6d, 0x47, 0x58}
#define AUTHOR_CHARACTERISTIC_UUID								{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd5, 0x6d, 0x47, 0x58}
#define AUTHOR_DESCRIPTOR_UUID									{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd6, 0x6d, 0x47, 0x58}
#define TECHNICAL_IMPLEMENTATION_CHARACTERISTIC_UUID			{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd7, 0x6d, 0x47, 0x58}
#define TECHNICAL_IMPLEMENTATION_DESCRIPTOR_UUID				{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd8, 0x6d, 0x47, 0x58}
#define VERSION_CHARACTERISTIC_UUID								{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xd9, 0x6d, 0x47, 0x58}
#define VERSION_DESCRIPTOR_UUID									{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xda, 0x6d, 0x47, 0x58}
#define DEVICE_NAME_CHARACTERISTIC_UUID							{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xdb, 0x6d, 0x47, 0x58}
#define DEVICE_NAME_DESCRIPTOR_UUID								{0x19, 0x24, 0xe0, 0xc0, 0xc0, 0x05, 0x84, 0x9a, 0xe0, 0x4a, 0x53, 0xcc, 0xdc, 0x6d, 0x47, 0x58}

#define DATA_SERVICE_UUID										{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xf7, 0x3d, 0x80, 0x07}
#define DATA_CHARACTERISTIC_UUID								{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xf8, 0x3d, 0x80, 0x07}
#define DATA_DESCRIPTOR_UUID									{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xf9, 0x3d, 0x80, 0x07}
#define DATA_DESCRIPTION_DESCRIPTOR_UUID						{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xfa, 0x3d, 0x80, 0x07}
#define TRANSFER_CHARACTERISTIC_UUID							{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xfb, 0x3d, 0x80, 0x07}
#define TRANSFER_DESCRIPTOR_UUID								{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xfc, 0x3d, 0x80, 0x07}
#define PLAYER_CHARACTERISTIC_UUID								{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xfd, 0x3d, 0x80, 0x07}
#define PLAYER_DESCRIPTOR_UUID									{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xfe, 0x3d, 0x80, 0x07}
#define LEFT_CHANNEL_CHARACTERISTIC_UUID						{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xf5, 0x3d, 0x80, 0x07}
#define LEFT_CHANNEL_DESCRIPTOR_UUID							{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xf6, 0x3d, 0x80, 0x07}
#define READY_CHARACTERISTIC_UUID								{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xf3, 0x3d, 0x80, 0x07}
#define READY_DESCRIPTOR_UUID									{0xf2, 0xa0, 0x59, 0x9e, 0xe8, 0xba, 0xa5, 0xb7, 0x4b, 0x4c, 0x40, 0xc7, 0xf4, 0x3d, 0x80, 0x07}

#define GATTS_DEMO_CHAR_VAL_LEN_MAX 0x40

static uint8_t vSerialNumberD[] = {'S', 'e', 'r', 'i', 'a', 'l', ' ', 'n', 'u', 'm', 'b', 'e', 'r'};
static uint8_t vSerialNumberC[] = {'0', '0', '0', '0', '0', '0', '0', '0', '0', '1'};
static uint8_t vChannelD[] = {'C', 'h', 'a', 'n', 'n', 'e', 'l'};
static uint8_t vChannelC[] = {'R', 'i', 'g', 'h', 't'};
static uint8_t vAuthorD[] = {'T', 'h', 'e', ' ', 'a', 'u', 't', 'h', 'o', 'r', ' ', 'o', 'f', ' ', 'i', 'd', 'e', 'a'};
static uint8_t vAuthorC[] = {'P', 'i', 'o', 't', 'r', ' ', 'P', 'e', 's', 'z', 'a', 't'};
static uint8_t vTechnicalImplementationD[] = {'T', 'e', 'c', 'h', 'n', 'i', 'c', 'a', 'l', ' ', 'i', 'm', 'p', 'l', 'e', 'm', 'e', 'n', 't', 'a', 't', 'i', 'o', 'n'};
static uint8_t vTechnicalImplementationC[] = {'V', 'l', 'a', 'd', 'i', 'm', 'i', 'r', ' ', 'M', 'i', 'l', 'a', 'n', 'o', 'v'};
static uint8_t vVersionD[] = {'V', 'e', 'r', 's', 'i', 'o', 'n'};
static uint8_t vVersionC[] = {'0', '0', '0', '0', '1'};
static uint8_t vDeviceNameD[] = {'D', 'e', 'v', 'i', 'c', 'e', ' ', 'n', 'a', 'm', 'e'};
static uint8_t vDeviceNameC[] = {'K', 'a', 'm', 'i', 'z', 'e', 'l', 'k', 'a'};
static uint8_t vDataD[] = {'D', 'a', 't', 'a'};
static uint8_t vDataDescriptionD[] = {'1', '3', ' ', 'm', 'o', 't', 'o', 'r', 's', ' ', 'b', 'y', ' ', 'o', 'n', 'e', ' ', 'b', 'y', 't', 'e', ' ', 'v', 'a', 'l', 'u', 'e'};
//static uint8_t vDataC[] = {0x00};
static uint8_t vTransferC[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
static uint8_t vTransferD[] = {'T', 'r', 'a', 'n', 's', 'f', 'e', 'r', ' ', 'd', 'a', 't', 'a', ' ', 't', 'o', ' ', 't', 'h', 'a', 't', ' ', 'd', 'e', 'v', 'i', 'c', 'e', ' ', 'a', 'd', 'd', 'r', 'e', 's', 's'};
static uint8_t vPlayerC[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
static uint8_t vPlayerD[] = {'P', 'l', 'a', 'y', 'e', 'r', ' ', 'd', 'e', 'v', 'i', 'c', 'e', ' ', 'a', 'd', 'd', 'r', 'e', 's', 's'};
static uint8_t vLeftChannelC[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
static uint8_t vLeftChannelD[] = {'L', 'e', 'f', 't', ' ', 'c', 'h', 'a', 'n', 'n', 'e', 'l', ' ', 'd', 'e', 'v', 'i', 'c', 'e', ' ', 'a', 'd', 'd', 'r', 'e', 's', 's'};
static uint8_t vReadyC[] = {0x00};
static uint8_t vReadyD[] = {'R', 'e', 'a', 'd', 'y', ' ', 't', 'o', ' ', 'w', 'o', 'r', 'k'};


static esp_attr_value_t serialNumberDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vSerialNumberD), .attr_value = vSerialNumberD};
static esp_attr_value_t serialNumberCharacteristicValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vSerialNumberC), .attr_value = vSerialNumberC};
static esp_attr_value_t channelDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vChannelD), .attr_value = vChannelD};
static esp_attr_value_t channelCharacteristicValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vChannelC), vChannelC};
static esp_attr_value_t authorDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vAuthorD), vAuthorD};
static esp_attr_value_t authorCharacteristicValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vAuthorC), vAuthorC};
static esp_attr_value_t technicalImplementationDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vTechnicalImplementationD), vTechnicalImplementationD};
static esp_attr_value_t technicalImplementationCharacteristicValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vTechnicalImplementationC), vTechnicalImplementationC};
static esp_attr_value_t versionDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vVersionD), vVersionD};
static esp_attr_value_t versionCharacteristicValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vVersionC), vVersionC};
static esp_attr_value_t deviceNameDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vDeviceNameD), vDeviceNameD};
static esp_attr_value_t deviceNameCharacteristicValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vDeviceNameC), vDeviceNameC};
static esp_attr_value_t dataDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vDataD), vDataD};
static esp_attr_value_t dataDescriptionDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vDataDescriptionD), vDataDescriptionD};
static esp_attr_value_t dataCharacteristicValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = 0, NULL};
static esp_attr_value_t transferCharacteristicValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vTransferC), vTransferC};
static esp_attr_value_t transferDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vTransferD), vTransferD};
static esp_attr_value_t playerCharacteristicValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vPlayerC), vPlayerC};
static esp_attr_value_t playerDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vPlayerD), vPlayerD};
static esp_attr_value_t leftChannelCharacteristicValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vLeftChannelC), vLeftChannelC};
static esp_attr_value_t leftChannelDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vLeftChannelD), vLeftChannelD};
static esp_attr_value_t readyCharacteristicValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vReadyC), vReadyC};
static esp_attr_value_t readyDescriptorValue = {.attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX, .attr_len = sizeof(vReadyD), vReadyD};

enum Services{
	INFO_SERVICE = 0,
	DATA_SERVICE = 1,
	NUMBER_OF_SERVICES
};

enum CharacteristicsOfInfoService{
	SERIAL_NUMBER_CHARACTERISTIC = 0,
	CHANNEL_CHARACTERISTIC = 1,
	AUTHOR_CHARACTERISTIC = 2,
	TECHNICAL_IMPLEMENTATION = 3,
	VERSION_CHARACTERISTIC = 4,
	DEVICE_NAME_CHARACTERISTIC = 5,
	NUMBER_OF_CHARACTERISTICS_OF_INFO_SERVICE
};

enum CharacteristicsOfDataService{
	DATA_CHARACTERISTIC = 0,
	TRANSFER_DEVICE_ADDRESS_CHARACTERISTIC = 1,
	PLAYER_DEVICE_ADDRESS_CHARACTERISTIC = 2,
	LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC = 3,
	READY_CHARACTERISTIC = 4,
	NUMBER_OF_CHARACTERISTICS_OF_DATA_SERVICE
};

static void gatts_profile_a_event_handler(esp_gatts_cb_event_t event, esp_gatt_if_t gatts_if, esp_ble_gatts_cb_param_t *param);

static bool isUUIDequal(esp_bt_uuid_t a, esp_bt_uuid_t b);
static int charValue = 0;

static esp_attr_control_t autoresp = {.auto_rsp = ESP_GATT_AUTO_RSP};

//======================================= HELPERS
// Инициализация библиотеки энергонезависимой памяти.
inline void hFlashInit(esp_err_t ret){
	if (ret == ESP_ERR_NVS_NO_FREE_PAGES || ret == ESP_ERR_NVS_NEW_VERSION_FOUND) {
		ESP_ERROR_CHECK(nvs_flash_erase());
		ret = nvs_flash_init();
	}
	ESP_ERROR_CHECK(ret);
}
inline void hCheckErr(esp_err_t ret, const char* str, const char* h__func__){
	if (ret) {
		ESP_LOGE(GATTS_TAG, "%s, %s: %s\n", h__func__, str, esp_err_to_name(ret));
		return;
}}
inline void hCheckErr2(esp_err_t ret, char* str){
	if (ret) {
		ESP_LOGE(GATTS_TAG, "%s, error code = %x\n", str, ret);
		return;
}}
void logMessengeUUID(const char* messengeSrting, const char* firstStr, uint16_t firstValue, const char* seconfString, uint16_t secondValue, const char* uuidString,
		esp_bt_uuid_t u, const char* thirdString, uint16_t thirdValue){ //, const char* fourthString, uint16_t fourthValur, const char fifthString, uint16_t fifthValur){
	char uuidValue[64];
	switch(u.len){
		case ESP_UUID_LEN_16:
			sprintf(uuidValue, "%d", u.uuid.uuid16);
			break;
		case ESP_UUID_LEN_32:
			sprintf(uuidValue, "%d", u.uuid.uuid32);
			break;
		case ESP_UUID_LEN_128:
			sprintf(uuidValue, "%1X %1X %1X %1X %1X %1X %1X %1X %1X %1X %1X %1X %1X %1X %1X %1X", u.uuid.uuid128[0], u.uuid.uuid128[1], u.uuid.uuid128[2], u.uuid.uuid128[3], u.uuid.uuid128[4], u.uuid.uuid128[5],
					u.uuid.uuid128[6], u.uuid.uuid128[7], u.uuid.uuid128[8], u.uuid.uuid128[9], u.uuid.uuid128[10], u.uuid.uuid128[11], u.uuid.uuid128[12], u.uuid.uuid128[13], u.uuid.uuid128[14], u.uuid.uuid128[15]);
		}
	if(strcmp (firstStr, "status")== 0  && firstValue == 0)
		if(thirdString == NULL)
				ESP_LOGI(GATTS_TAG, "%s, %s: %s, %s: %d, %s: %s\n", messengeSrting, firstStr, "GATT OK", seconfString, secondValue, uuidString, uuidValue);
		else
			ESP_LOGI(GATTS_TAG, "%s, %s: %s, %s: %d, %s: %s, %s: %d\n", messengeSrting, firstStr, "GATT OK", seconfString, secondValue, uuidString, uuidValue, thirdString, thirdValue);
//	else if(seconfString == NULL)
//		ESP_LOGI(GATTS_TAG, "%s, %s: %d, %s: %s\n", messengeSrting, firstStr, firstValue, uuidString, uuidValue);
	else if(thirdString == NULL)
		ESP_LOGI(GATTS_TAG, "%s, %s: %d, %s: %d, %s: %s\n", messengeSrting, firstStr, firstValue, seconfString, secondValue, uuidString, uuidValue);
	else
		ESP_LOGI(GATTS_TAG, "%s, %s: %d, %s: %d, %s: %s, %s: %d\n", messengeSrting, firstStr, firstValue, seconfString, secondValue, uuidString, uuidValue, thirdString, thirdValue);
}
//===============================================

typedef struct descriptor_handle{					// Дескриптор.
	uint16_t descr_handle;							// Указатель дескриптора.
	esp_gatt_perm_t perm;							// Свойства харрактеристики.
	esp_bt_uuid_t descr_uuid;						// UUID дескриптора.
	esp_attr_value_t* value;						// Значение дескриптора.
	esp_attr_control_t* autoResponse;				// Флаг автоответа. NULL означает, что отвечать надо вручную, иначе можно было бы использовать ESP_GATT_RSP_BY_APP, что и является ответом.
} descriptor_handle;
typedef struct characteristic_handle{				// Харрактеристика.
	uint16_t char_handle;							// Указатель харрактеристики.
	esp_bt_uuid_t char_uuid;						// uuid характеристики.
	esp_gatt_perm_t perm;							// Свойства харрактеристики.
	esp_gatt_char_prop_t property;					// Свойства значения харрактеристики.
	uint16_t numOfDescriptors;						// количество дескрипторов.
	descriptor_handle* descrs;						// Массив дескрипторов.
	esp_attr_value_t* value;						// Значение характеристики.
	esp_attr_control_t* autoResponse;				// Флаг автоответа. NULL означает, что отвечать надо вручную, иначе можно было бы использовать ESP_GATT_RSP_BY_APP, что и является ответом.
} characteristic_handle;
typedef struct service_holder{						// Сервис.
	uint16_t serv_handle;							// Указатель сервиса.
	esp_gatt_srvc_id_t service_id;					// ID сервиса, содержит uuid и instance id.
	uint16_t numOfChars;							// Количество харрактеристик.
	characteristic_handle* chars;					// Массив харрактеристик.
} service_holder;
typedef struct gatts_profile_inst{					// Профиль.
		esp_gatts_cb_t gatts_cb;					// Обработчик событий.
		uint16_t gatts_if;							// Интерфейс GATTS.
		uint16_t app_id;							// ID профиля (типа PROFILE_A_APP_ID).
		uint16_t conn_id;							// ID соединения.
		uint16_t numOfServs;						// Количество сервисов.
		service_holder* servs;						// Массив сервисов.
}gatts_profile_inst;

/* Таблица профилей. Каждому приложению соответствует один профиль и один интерфейс gatts_if,
 * который автоматически назначается событием ESP_GATTS_REG_EVT. Также здесь хранятся ссылки на обработчики событий.
 * Таблица заполняется вручную. PROFILE_A_APP_ID - порядковый номер профиля.
 * Особо надо следить, чтобы добавляемые атрибуты отличались. У одного сервиса не может быть двух характеристик
 * с тем же самым UUID. Также как и у одной характеристики не может быть двух сервисов с одинаковым UUID.
 * Сервисы, дескрипторы и харрактеристики в таблицах обязательно должны быть расположены с 0-го индекса.  */
struct gatts_profile_inst gl_profile_tab[PROFILE_NUM] = {
	[PROFILE_A_APP_ID] = {																							// Профиль.
		.gatts_cb = gatts_profile_a_event_handler,																	// Обработчик событий профиля A.
		.gatts_if = ESP_GATT_IF_NONE,	   																			// Начальное значение ESP_GATT_IF_NONE, так как ещё не зарегистрирован ни один профиль.
		.app_id = PROFILE_A_APP_ID,																					// ID профиля (типа PROFILE_A_APP_ID).
		.numOfServs = 2,																							// Количество сервисов.
		.servs = (service_holder[NUMBER_OF_SERVICES]){																				// Сервисы.
			[INFO_SERVICE] = {
					.service_id = {.id = {.uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = INFO_SERVICE_UUID}}, .inst_id = 0x00}, .is_primary = true},							// ID сервиса.
					.numOfChars = NUMBER_OF_CHARACTERISTICS_OF_INFO_SERVICE,																				// Количество характеристик.
					.chars = (characteristic_handle[NUMBER_OF_CHARACTERISTICS_OF_INFO_SERVICE]){															// Характеристики.
						[SERIAL_NUMBER_CHARACTERISTIC] = {
								.char_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = SERIAL_NUMBER_CHARACTERISTIC_UUID}},					// UUID характеристики.
								.perm = ESP_GATT_PERM_READ,															// разрешены чтение и запись значения харрактеристики.
								.property = ESP_GATT_CHAR_PROP_BIT_READ,											// Разрешены запись, чтение и уведомления самой харрактеристики, нужны для клиента.
								.value = &serialNumberCharacteristicValue,											// Значение харрактеристики.
								.autoResponse = NULL,																// Автоответ.
								.numOfDescriptors = 1,																// Количество дискриптора.
								.descrs = (descriptor_handle[1]) {													// Дискрипторы.
									[0] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = SERIAL_NUMBER_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ, .value = &serialNumberDescriptorValue, .autoResponse = NULL},	// Дискриптор.
						}},
						[CHANNEL_CHARACTERISTIC] = {
								.char_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = CHANNEL_CHARACTERISTIC_UUID}},						// UUID характеристики.
								.perm = ESP_GATT_PERM_READ,															// разрешены чтение и запись значения харрактеристики.
								.property = ESP_GATT_CHAR_PROP_BIT_READ,											// Разрешены запись, чтение и уведомления самой харрактеристики, нужны для клиента.
								.value = &channelCharacteristicValue,												// Значение харрактеристики.
								.autoResponse = NULL,																// Автоответ.
								.numOfDescriptors = 1,																// Количество дискриптора.
								.descrs = (descriptor_handle[1]) {													// Дискрипторы.
									[0] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = CHANNEL_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ, .value = &channelDescriptorValue, .autoResponse = NULL},	// Дискриптор.
						}},
						[AUTHOR_CHARACTERISTIC] = {
								.char_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = AUTHOR_CHARACTERISTIC_UUID}},						// UUID характеристики.
								.perm = ESP_GATT_PERM_READ,															// разрешены чтение и запись значения харрактеристики.
								.property = ESP_GATT_CHAR_PROP_BIT_READ,											// Разрешены запись, чтение и уведомления самой харрактеристики, нужны для клиента.
								.value = &authorCharacteristicValue,												// Значение харрактеристики.
								.autoResponse = NULL,																// Автоответ.
								.numOfDescriptors = 1,																// Количество дискриптора.
								.descrs = (descriptor_handle[1]) {													// Дискрипторы.
									[0] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = AUTHOR_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ, .value = &authorDescriptorValue, .autoResponse = NULL},	// Дискриптор.
						}},
						[TECHNICAL_IMPLEMENTATION] = {
								.char_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = TECHNICAL_IMPLEMENTATION_CHARACTERISTIC_UUID}},		// UUID характеристики.
								.perm = ESP_GATT_PERM_READ,															// разрешены чтение и запись значения харрактеристики.
								.property = ESP_GATT_CHAR_PROP_BIT_READ,											// Разрешены запись, чтение и уведомления самой харрактеристики, нужны для клиента.
								.value = &technicalImplementationCharacteristicValue,								// Значение харрактеристики.
								.autoResponse = NULL,																// Автоответ.
								.numOfDescriptors = 1,																// Количество дискриптора.
								.descrs = (descriptor_handle[1]) {													// Дискрипторы.
									[0] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = TECHNICAL_IMPLEMENTATION_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ, .value = &technicalImplementationDescriptorValue, .autoResponse = NULL},	// Дискриптор.
						}},
						[VERSION_CHARACTERISTIC] = {
								.char_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = VERSION_CHARACTERISTIC_UUID}},						// UUID характеристики.
								.perm = ESP_GATT_PERM_READ,															// разрешены чтение и запись значения харрактеристики.
								.property = ESP_GATT_CHAR_PROP_BIT_READ,											// Разрешены запись, чтение и уведомления самой харрактеристики, нужны для клиента.
								.value = &versionCharacteristicValue,												// Значение харрактеристики.
								.autoResponse = NULL,																// Автоответ.
								.numOfDescriptors = 1,																// Количество дискриптора.
								.descrs = (descriptor_handle[1]) {													// Дискрипторы.
									[0] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = VERSION_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ, .value = &versionDescriptorValue, .autoResponse = NULL},	// Дискриптор.
						}},
						[DEVICE_NAME_CHARACTERISTIC] = {
								.char_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = DEVICE_NAME_CHARACTERISTIC_UUID}},					// UUID характеристики.
								.perm = ESP_GATT_PERM_READ,															// разрешены чтение и запись значения харрактеристики.
								.property = ESP_GATT_CHAR_PROP_BIT_READ,											// Разрешены запись, чтение и уведомления самой харрактеристики, нужны для клиента.
								.value = &deviceNameCharacteristicValue,											// Значение харрактеристики.
								.autoResponse = NULL,																// Автоответ.
								.numOfDescriptors = 1,																// Количество дискриптора.
								.descrs = (descriptor_handle[1]) {													// Дискрипторы.
									[0] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = DEVICE_NAME_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ, .value = &deviceNameDescriptorValue, .autoResponse = NULL},	// Дискриптор.
						}},
			}},
			[DATA_SERVICE] = {
					.service_id = {{{ESP_UUID_LEN_128, .uuid = {.uuid128 = DATA_SERVICE_UUID}}, 0x00}, true},		// ID сервиса.
					.numOfChars = NUMBER_OF_CHARACTERISTICS_OF_DATA_SERVICE,																				// Количество характеристик.
					.chars = (characteristic_handle[NUMBER_OF_CHARACTERISTICS_OF_DATA_SERVICE]){															// Характеристики.
						[DATA_CHARACTERISTIC] = {
								.char_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = DATA_CHARACTERISTIC_UUID}},							// UUID характеристики.
								.perm = ESP_GATT_PERM_READ | ESP_GATT_PERM_WRITE,									// разрешены чтение и запись значения харрактеристики.
								.property = ESP_GATT_CHAR_PROP_BIT_READ | ESP_GATT_CHAR_PROP_BIT_WRITE | ESP_GATT_CHAR_PROP_BIT_NOTIFY,	// Разрешены запись, чтение и уведомления самой харрактеристики, нужны для клиента.
								.value = &dataCharacteristicValue,													// Значение харрактеристики.
								.autoResponse = NULL,																// Автоответ.
								.numOfDescriptors = 2,																// Количество дискриптора.
								.descrs = (descriptor_handle[2]) {													// Дискрипторы.
									[0] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = DATA_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ | ESP_GATT_PERM_WRITE, .value = &dataDescriptorValue, .autoResponse = NULL},	// Дискриптор.
									[1] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = DATA_DESCRIPTION_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ | ESP_GATT_PERM_WRITE, .value = &dataDescriptionDescriptorValue, .autoResponse = NULL},	// Дискриптор.
						}},
						[TRANSFER_DEVICE_ADDRESS_CHARACTERISTIC] = {
								.char_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = TRANSFER_CHARACTERISTIC_UUID}},						// UUID характеристики.
								.perm = ESP_GATT_PERM_READ | ESP_GATT_PERM_WRITE,									// разрешены чтение и запись значения харрактеристики.
								.property = ESP_GATT_CHAR_PROP_BIT_READ | ESP_GATT_CHAR_PROP_BIT_WRITE | ESP_GATT_CHAR_PROP_BIT_NOTIFY,	// Разрешены запись, чтение и уведомления самой харрактеристики, нужны для клиента.
								.value = &transferCharacteristicValue,												// Значение харрактеристики.
								.autoResponse = NULL,																// Автоответ.
								.numOfDescriptors = 1,																// Количество дискриптора.
								.descrs = (descriptor_handle[1]) {													// Дискрипторы.
									[0] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = TRANSFER_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ | ESP_GATT_PERM_WRITE, .value = &transferDescriptorValue, .autoResponse = NULL},	// Дискриптор.
						}},
						[PLAYER_DEVICE_ADDRESS_CHARACTERISTIC] = {
								.char_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = PLAYER_CHARACTERISTIC_UUID}},						// UUID характеристики.
								.perm = ESP_GATT_PERM_READ | ESP_GATT_PERM_WRITE,									// разрешены чтение и запись значения харрактеристики.
								.property = ESP_GATT_CHAR_PROP_BIT_READ | ESP_GATT_CHAR_PROP_BIT_WRITE | ESP_GATT_CHAR_PROP_BIT_NOTIFY,	// Разрешены запись, чтение и уведомления самой харрактеристики, нужны для клиента.
								.value = &playerCharacteristicValue,												// Значение харрактеристики.
								.autoResponse = NULL,																// Автоответ.
								.numOfDescriptors = 1,																// Количество дискриптора.
								.descrs = (descriptor_handle[1]) {													// Дискрипторы.
									[0] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = PLAYER_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ | ESP_GATT_PERM_WRITE, .value = &playerDescriptorValue, .autoResponse = NULL},	// Дискриптор.
						}},
						[LEFT_CHANNEL_DEVICE_ADDRESS_CHARACTERISTIC] = {
								.char_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = LEFT_CHANNEL_CHARACTERISTIC_UUID}},						// UUID характеристики.
								.perm = ESP_GATT_PERM_READ | ESP_GATT_PERM_WRITE,									// разрешены чтение и запись значения харрактеристики.
								.property = ESP_GATT_CHAR_PROP_BIT_READ | ESP_GATT_CHAR_PROP_BIT_WRITE | ESP_GATT_CHAR_PROP_BIT_NOTIFY,	// Разрешены запись, чтение и уведомления самой харрактеристики, нужны для клиента.
								.value = &leftChannelCharacteristicValue,											// Значение харрактеристики.
								.autoResponse = NULL,																// Автоответ.
								.numOfDescriptors = 1,																// Количество дискриптора.
								.descrs = (descriptor_handle[1]) {													// Дискрипторы.
									[0] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = LEFT_CHANNEL_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ | ESP_GATT_PERM_WRITE, .value = &leftChannelDescriptorValue, .autoResponse = NULL},	// Дискриптор.
						}},
						[READY_CHARACTERISTIC] = {
								.char_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = READY_CHARACTERISTIC_UUID}},						// UUID характеристики.
								.perm = ESP_GATT_PERM_READ | ESP_GATT_PERM_WRITE,									// разрешены чтение и запись значения харрактеристики.
								.property = ESP_GATT_CHAR_PROP_BIT_READ | ESP_GATT_CHAR_PROP_BIT_WRITE | ESP_GATT_CHAR_PROP_BIT_NOTIFY,	// Разрешены запись, чтение и уведомления самой харрактеристики, нужны для клиента.
								.value = &readyCharacteristicValue,													// Значение харрактеристики.
								.autoResponse = NULL,																// Автоответ.
								.numOfDescriptors = 1,																// Количество дискриптора.
								.descrs = (descriptor_handle[1]) {													// Дискрипторы.
									[0] = {.descr_uuid = {.len = ESP_UUID_LEN_128, .uuid = {.uuid128 = READY_DESCRIPTOR_UUID}}, .perm = ESP_GATT_PERM_READ | ESP_GATT_PERM_WRITE, .value = &readyDescriptorValue, .autoResponse = NULL},	// Дискриптор.
						}},
			}},
	}},
};

void profilesInit(esp_gatts_cb_event_t event, esp_ble_gatts_cb_param_t *param, esp_gatt_if_t gatts_if);
unsigned short getProfileID(esp_gatt_if_t gatts_if);
inline void createService(uint16_t profileID);
// Поиск места в таблице профилей сервиса по его UUID. Функция возвращает или индекс в таблице, или -1.
__int16_t findIndexOfServiceByUUID(esp_gatt_id_t idx, uint16_t profileID){
	for(int i = 0; i < gl_profile_tab[profileID].numOfServs; ++i)													// Перебираем все сервисы в данном профиле.
		if(isUUIDequal(gl_profile_tab[profileID].servs[i].service_id.id.uuid, idx.uuid) && gl_profile_tab[profileID].servs[i].service_id.id.inst_id == idx.inst_id) // Если нашли такой, UUID которого совпадает с тем, что ищем.
			return i;																								// Возвращаем его место в таблице.
	logMessengeUUID("Can not find service by UUID", "profileID", profileID, NULL, 0, "service UUID", idx.uuid, NULL, 0);
	return -1;																										// Если не нашли, возвращаем -1.
}
// Поиск места в таблице профилей сервиса по его указателю атрибута. Функция возвращает или индекс в таблице, или -1.
__int16_t findIndexOfServiceByHandle(uint16_t handle, uint16_t profileID){
	for(int i = 0; i < gl_profile_tab[profileID].numOfServs; ++i)													// Перебираем все сервисы в данном профиле.
		if(gl_profile_tab[profileID].servs[i].serv_handle == handle)												// Если нашли такой, указатель атрибута которого совпадает с тем, что ищем.
			return i;																								// Возвращаем его место в таблице.
	ESP_LOGE(GATTS_TAG, "Can not find service by handle; serv_handle: %i, profileID: %i!\n", handle, profileID);
	return -1;																										// Если не нашли, возвращаем -1.
}
// Поиск места в таблице профилей характеристики по его UUID. Функция возвращает или индекс в таблице, или -1.
__int16_t findIndexOfCharacteristicByUUID(uint16_t profileID, __int16_t index_service, esp_bt_uuid_t charUuid){
	if(index_service == -1) return -1;
	for(int j = 0; j < gl_profile_tab[profileID].servs[index_service].numOfChars; ++j)								// Перебираем все характеристики в данном профиле.
		if(isUUIDequal(gl_profile_tab[profileID].servs[index_service].chars[j].char_uuid, charUuid))				// Если нашли такую, UUID которого совпадает с тем, что ищем.
			return j;																								// Возвращаем его место в таблице.
	logMessengeUUID("Can not find characteristic by UUID", "profileID", profileID, NULL, 0, "characteristic UUID", charUuid, NULL, 0);
	return -1;																										// Если не нашли, возвращаем -1.
}
// Поиск места в таблице профилей дескриптора по его UUID. Функция возвращает или индекс в таблице, или -1.
__int16_t findIndexOfDescriptorByUUID(uint16_t profileID, __int16_t index_service, __int16_t index_char, esp_bt_uuid_t descrUuid){
	if(index_service == -1 || index_char == -1) return -1;
	for(int k = 0; k < gl_profile_tab[profileID].servs[index_service].chars[index_char].numOfDescriptors; ++k)		// Перебираем все дескрипторы в данном профиле.
		if(isUUIDequal(gl_profile_tab[profileID].servs[index_service].chars[index_char].descrs[k].descr_uuid, descrUuid))// Если нашли такой, UUID которого совпадает с тем, что ищем.
			return k;																								// Возвращаем его место в таблице.
	logMessengeUUID("Can not find descriptor by UUID", "profileID", profileID, "index service", index_service, "descriptor UUID", descrUuid, "index characteristic", index_char);
	return -1;																										// Если не нашли, возвращаем -1.
}
// Добавление харрактеристики профилю.
void addChar(uint16_t profileID, __int16_t index_service){
	for (int j = 0; j < gl_profile_tab[profileID].servs[index_service].numOfChars; ++j){							// Перебираем все характеристики в данном профиле.
		if(gl_profile_tab[profileID].servs[index_service].chars[j].char_handle == 0){								// Если нашли ещё не добавленную харрактеристику, добавляем.
			esp_err_t ret = esp_ble_gatts_add_char(gl_profile_tab[profileID].servs[index_service].serv_handle, &gl_profile_tab[profileID].servs[index_service].chars[j].char_uuid, gl_profile_tab[profileID].servs[index_service].chars[j].perm,
					 gl_profile_tab[profileID].servs[index_service].chars[j].property, gl_profile_tab[profileID].servs[index_service].chars[j].value, gl_profile_tab[profileID].servs[index_service].chars[j].autoResponse);
			hCheckErr2(ret, "add char failed");
			return;																									// Нельзя добавлять все характеристики сразу. Надо дождаться события, чтобы записать указатель атрибута.
	}}
	createService(profileID);																						// После того, как все характеристики окажутся добавленны, возвращаемся на уровень сервисов.
}

void registerDinamicVlaues(){
	uint8_t u = 0;
	gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value = (uint8_t*) malloc(sizeof(u) * sizeof(uint8_t));
	memcpy(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value, &u, sizeof(u));
	gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_len = sizeof(u);
	gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_max_len = GATTS_DEMO_CHAR_VAL_LEN_MAX;
//	uint8_t ui[5] = {0, 1, 2, 3, 4};
//	memcpy(gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_value, &ui, sizeof(ui));
//	gl_profile_tab[PROFILE_A_APP_ID].servs[DATA_SERVICE].chars[DATA_CHARACTERISTIC].value->attr_len = sizeof(u);

}


// Регистрация профилей.
void addProfiles(){
	for(int i = 0; i < PROFILE_NUM; ++i){																			// Устанавливаем все указатели атрибутов в неинициализированное состояние.
		for(int j = 0; j < gl_profile_tab[i].numOfServs; ++j){
			gl_profile_tab[i].servs[j].serv_handle = 0;
			for(int k = 0; k < gl_profile_tab[i].servs[j].numOfChars; ++k){
				gl_profile_tab[i].servs[j].chars[k].char_handle = 0;
				for(int l = 0; l < gl_profile_tab[i].servs[j].chars[k].numOfDescriptors; ++l){
					gl_profile_tab[i].servs[j].chars[k].descrs[l].descr_handle = 0;
	}}}}
	for(int i = 0; i < PROFILE_NUM; ++i){
		hCheckErr2(esp_ble_gatts_app_register(i), "gatts app register error");										// Регистрируем профили приложения.
}}
// Добавление дескриптора харрактеристике.
void addDescroptor(uint16_t profileID, __int16_t index_service, __int16_t index_char){
	for(int k = 0; k < gl_profile_tab[profileID].servs[index_service].chars[index_char].numOfDescriptors; ++k){		// Перебираем все дескрипторы данной харрактеристики.
		if(gl_profile_tab[profileID].servs[index_service].chars[index_char].descrs[k].descr_handle == 0){			// Если нашли ещё не добавленный, добавляем.
			esp_err_t ret = esp_ble_gatts_add_char_descr(gl_profile_tab[profileID].servs[index_service].serv_handle, &gl_profile_tab[profileID].servs[index_service].chars[index_char].descrs[k].descr_uuid,
										gl_profile_tab[profileID].servs[index_service].chars[index_char].descrs[k].perm, NULL, NULL);
			hCheckErr2(ret, "add descriptor failed");
			return;																									// Нельзя добавлять все дескрипторы сразу. Надо дождаться события, чтобы записать указатель атрибута.
	}}
	addChar(profileID, index_service);																				// Если все дескрипторы уже добавлены, возвращаемся на уровень характеристик.
}
// Функция регистрации таблицы профилей.
void profilesInit(esp_gatts_cb_event_t event, esp_ble_gatts_cb_param_t *param, esp_gatt_if_t gatts_if){
	uint16_t profileID = getProfileID(gatts_if);																	// Находим ID профиля по его GATT интерфейсу.
	switch(event){
	case ESP_GATTS_REG_EVT:																							// Профиль зарегистрирован, добавляем сервисы.
		if(param->reg.status != ESP_GATT_OK){
			ESP_LOGE(GATTS_TAG, "Profile was registered (REGISTER_APP_EVT); status: FILED, profileID: %d\n", param->reg.app_id);
			return;
		}
		ESP_LOGI(GATTS_TAG, "Profile was registered (REGISTER_APP_EVT); status: GATT OK, profileID: %d\n", param->reg.app_id);
		createService(profileID);																					// Количество харрактеристик в профиле: 1 + (nofCharacteristics() * 3).
		break;
	case ESP_GATTS_CREATE_EVT:{																						// Сервисы созданы, добавляем харрактеристики.
		logMessengeUUID("Service was created (CREATE_SERVICE_EVT)", "status", param->create.status, "handle", param->create.service_handle, "UUID", param->create.service_id.id.uuid, NULL, 0);
		if(param->create.status != ESP_GATT_OK) return;
		__int16_t i = findIndexOfServiceByUUID(param->create.service_id.id, profileID);								// Находим нужный сервис по его GATT интерфейсу.
		if(i == -1) return;
		gl_profile_tab[profileID].servs[i].serv_handle = param->create.service_handle;								// Записываем указатель атрибута службы (номер в таблице).
		esp_ble_gatts_start_service(gl_profile_tab[profileID].servs[i].serv_handle);								// Запускаем сервис.
		addChar(profileID, i);																						// После запуска сервиса надо добавить его харрактеристики.
		} break;
	case ESP_GATTS_ADD_CHAR_EVT:{																					// Харрактеристики добавлены, добавляем дискрипторы.
		logMessengeUUID("Charactreristic was added (ADD_CHAR_EVT)", "status", param->add_char.status, "handle", param->add_char.attr_handle, "characteristic UUID", param->add_char.char_uuid, "handle", param->add_char.attr_handle);
		if(param->add_char.status != ESP_GATT_OK) return;
		__int16_t i = findIndexOfServiceByHandle(param->add_char.service_handle, profileID);						// Находим нужный сервис по его указателю атрибута.
		__int16_t j = findIndexOfCharacteristicByUUID(profileID, i, param->add_char.char_uuid);						// Находим нужный сервис по его UUID.
		if(j == -1) {
			ESP_LOGE(GATTS_TAG, "ERROR -1\n");
			return;
		}
		gl_profile_tab[profileID].servs[i].chars[j].char_handle = param->add_char.attr_handle;						// Записываем указатель атрибута характеристики.
		addDescroptor(profileID, i, j);																				// Добавляем дескрипторы.
		} break;
	case ESP_GATTS_ADD_CHAR_DESCR_EVT:{																				// Добавление дескриптора закончено.
		logMessengeUUID("Descriptor was added (ESP_GATTS_ADD_CHAR_DESCR_EVT)", "status", param->add_char_descr.status, "handle", param->add_char_descr.attr_handle, "UUID", param->add_char_descr.descr_uuid, "description handle", param->add_char_descr.attr_handle);
		if(param->add_char_descr.status != ESP_GATT_OK) return;
		__int16_t i = findIndexOfServiceByHandle(param->add_char_descr.service_handle, profileID);					// Находим нужный сервис по его указателю атрибута.
		__int16_t j;
		for(j = 0; j < gl_profile_tab[profileID].servs[i].numOfChars; ++j){											// Ищем характеристику к которой принадлежит дескриптор. Она должна быть последняя добавленная.
			if(gl_profile_tab[profileID].servs[i].chars[j].char_handle == 0){										// Если нашли недобавленную, значит нужная нам - предидущая.
				break;
		}}
		--j;
		if(j < 0) {
			ESP_LOGE(GATTS_TAG, "Something goes wrong.\n");
			return;
		}
		__int16_t k = findIndexOfDescriptorByUUID(profileID, i, j, param->add_char_descr.descr_uuid);				// Ищем нужный дескриптор по его UUID.
		gl_profile_tab[profileID].servs[i].chars[j].descrs[k].descr_handle = param->add_char_descr.attr_handle;		// Записываем уго указатель атрибута.
		addDescroptor(profileID, i, j);																				// Возвращаемся на этап добавления дескрипторов.
	}break;
	default:
		break;
}}
// Функция возвращает ID профиля по его GATT интерфейсу.
uint16_t getProfileID(esp_gatt_if_t gatts_if){
	for (int idx = 0; idx < PROFILE_NUM; idx++){																	// Проходим все профили.
		if (gatts_if == gl_profile_tab[idx].gatts_if){																// Если GATT интерфейс совпадает, возвращаем место в таблице.
			return idx;
	}}
	return 0;
}
// Создание сервиса.
inline void createService(uint16_t profileID){
	for(int i = 0; i < gl_profile_tab[profileID].numOfServs; ++i){													// Проходим все сервисы профиля.
		if(gl_profile_tab[profileID].servs[i].serv_handle == 0){													// Если нашли не добавленный профиль.
			uint16_t num = 0;																						// Общее количество атрибутов.
			for(int j = 0; j < gl_profile_tab[profileID].servs[i].numOfChars; ++j){									// Подсчитываем количество характеристик и дескрипторов сервиса.
				num += gl_profile_tab[profileID].servs[i].chars[j].numOfDescriptors;								// И нужное для них число указателей атрибута.
			}
			num += gl_profile_tab[profileID].servs[i].numOfChars * 2 + 1;											// Для каждой харрактеристики надо 2 указателя атрибута (handels), а для дескриптора 1.
			esp_err_t ret = esp_ble_gatts_create_service(gl_profile_tab[profileID].gatts_if, &gl_profile_tab[profileID].servs[i].service_id, num);	// Создаём сервис.
			if (ret)
				logMessengeUUID("Unable to create service", "profileID", profileID, NULL, 0, "service UUID ", gl_profile_tab[profileID].servs[i].service_id.id.uuid, esp_err_to_name(ret), 0);
			return;																									// Нельзя добавлять все сервисы сразу. Надо дождаться события, чтобы сохранить его указатель атрибута.
		}}
	ESP_LOGI(GATTS_TAG, "No left unregister services; profileID: %i\n", profileID);
}
// Сравнение UUID.
bool isUUIDequal(esp_bt_uuid_t a, esp_bt_uuid_t b){
	if(a.len != b.len) return false;
	switch(a.len){
		case ESP_UUID_LEN_16:  return a.uuid.uuid16 == b.uuid.uuid16;
		case ESP_UUID_LEN_32:  return a.uuid.uuid32 == b.uuid.uuid32;
		case ESP_UUID_LEN_128: return memcmp(a.uuid.uuid128, b.uuid.uuid128, ESP_UUID_LEN_128) == 0;
	}
	return false;
}

#endif /* MAIN_PROFILES_H_ */
