/*
 * pwm.h
 *
 *  Created on: 26 kwi 2021
 *      Author: Kashey
 */

#ifndef MAIN_PWM_H_
#define MAIN_PWM_H_

#include <stdio.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "driver/ledc.h"
#include "esp_err.h"
#include "esp_log.h"

#define LEDC_TEST_CH_NUM       (16)

ledc_channel_config_t ledc_channel[16];

void stopPWM(){

//	gpio_pad_select_gpio(5);
//	gpio_set_level(5, 0);

	for(int i = 0; i < 16; ++i){
		ledc_set_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel, 0);
		ledc_update_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel);
}}

void pwmInitialization(void){
    ledc_timer_config_t ledc_timer = {							// Конфирунация первого таймера (LEDC_TIMER_1), который будем связывать с каналами ledc.
        .duty_resolution = LEDC_TIMER_8_BIT,					// Количество ступеней заполнения цикла.
        .freq_hz = 100,										// Частота ШИМ сигнала.
        .speed_mode = LEDC_HIGH_SPEED_MODE,						// Скорость. Может быть высокая (железная реализация) или низкая (програмная реализация).
        .timer_num = LEDC_TIMER_0,								// Номер таймера, который конфигурируем.
        .clk_cfg = LEDC_AUTO_CLK,								// Auto select the source clock
    };
    ledc_timer_config(&ledc_timer);								// Устанавливаем конфигурацию первого таймера (LEDC_TIMER_0).
    ledc_timer.timer_num = LEDC_TIMER_1;
    ledc_timer.speed_mode = LEDC_LOW_SPEED_MODE;
    ledc_timer_config(&ledc_timer);								// Устанавливаем конфигурацию первого таймера (LEDC_TIMER_1).

    ledc_channel[0].gpio_num = 23;								// Номер вывода GPIO, ассоциированный с каналом.
    ledc_channel[1].gpio_num = 22;
    ledc_channel[2].gpio_num = 1;
    ledc_channel[3].gpio_num = 3;
    ledc_channel[4].gpio_num = 21;
    ledc_channel[5].gpio_num = 19;
    ledc_channel[6].gpio_num = 18;
    ledc_channel[7].gpio_num = 5;
    ledc_channel[8].gpio_num = 13;								// Номер вывода GPIO, ассоциированный с каналом.
    ledc_channel[9].gpio_num = 12;
    ledc_channel[10].gpio_num = 14;
    ledc_channel[11].gpio_num = 27;
    ledc_channel[12].gpio_num = 26;
    ledc_channel[13].gpio_num = 25;
    ledc_channel[14].gpio_num = 33;
    ledc_channel[15].gpio_num = 32;

    for(int i = 0; i < LEDC_TEST_CH_NUM/2; ++i){				// Подготавливаем конфигурацию каналов ledc.
    	ESP_LOGI(GATTS_TAG, "----------------------\n");
    	ledc_channel[i].channel = i;							// Номер канала.
    	ledc_channel[i].duty = 0;								// Длительность, для инициализации устанавливаем в 0. Может быть от 0 до (2**duty_resolution)).
    	ledc_channel[i].speed_mode = LEDC_HIGH_SPEED_MODE;		// Скорость, которую мы устанавливали для таймера (высокая или низкая).
    	ledc_channel[i].hpoint = 0;								// LEDC channel hpoint value, the max value is 0xfffff.
    	ledc_channel[i].timer_sel = LEDC_TIMER_0;				// Номер таймера, с которым мы хотим ассоциировать канал. Если несколько каналов ассоциированны с одним таймером,
    															// они должны иметь одинаковое количество ступеней заполнения цикла (freq_hz and duty_resolution).
    	ledc_channel[i+8].channel = i;							// Делаем то же самое для второй восьмёрки каналов.
    	ledc_channel[i+8].duty = 0;
    	ledc_channel[i+8].speed_mode = LEDC_LOW_SPEED_MODE;
    	ledc_channel[i+8].hpoint = 0;
    	ledc_channel[i+8].timer_sel = LEDC_TIMER_1;

    	printf("i = %i, speed mode = %i, ", i, ledc_channel[i].speed_mode);
    	ledc_channel_config(&ledc_channel[i]);					// Устанавливаем настройки каналов. В этот момент начинается генерация ШИМ на выводе.
    	printf("i + 8 = %i, speed mode = %i, ", i+8, ledc_channel[i+8].speed_mode);
    	ledc_channel_config(&ledc_channel[i+8]);
    	printf("- %i", i+8);
    	stopPWM();
    }
}
// Функция получает на вход массив длинной в 16 символов, которыми управляются вибромоторы. Порядок символов - номер мотора, число - мощность включения.
void runPWM(uint8_t* dataPWM){
	for(int i = 0; i < 16; ++i){
		ledc_set_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel, dataPWM[i]);

		/*
		switch(dataPWM[i] & 0b111){
		case 0:
			ledc_set_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel, 0);
			break;
		case 1:
			ledc_set_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel, 6);
			break;
		case 2:
			ledc_set_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel, 7);
			break;
		case 3:
			ledc_set_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel, 9);
			break;
		case 4:
			ledc_set_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel, 10);
			break;
		case 5:
			ledc_set_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel, 11);
			break;
		case 6:
			ledc_set_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel, 12);
			break;
		case 7:
			ledc_set_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel, 15);
			break;
		}
		*/
		ledc_update_duty(ledc_channel[i].speed_mode, ledc_channel[i].channel);
	}
	printf("\n");

    /*
     * 14 - 0
     * 12 - 1
     * 11 - 2
     * 10 - 3
     * 9 - 4
     * 8 - 5----
     * 7 - 6
     * 6 - 7
     *
     * 4 - 0
     **/

}

    // ledc_stop() - остановка генерации.
    // ledc_set_duty() - изменение заполнения. После изменения надо вызвать ledc_update_duty().
    // ledc_get_duty() - получение текущего заполнения.
    // Также для изменения заполнения можно использовать ledc_channel_config().
    // Для аппаратного затухания надо сначала инициировать его: ledc_fade_func_install(),
    // настроить с помощью ledc_set_fade_with_time(), ledc_set_fade_with_step(), ledc_set_fade() и инициировать с помощью ledc_fade_start().
    // Отключить затухание можно с помощью ledc_fade_func_uninstall().
    // Частоту можно изменить с помощью ledc_set_freq().
    // Проверить текущую частоту можно с помощью ledc_get_freq().
    // Поменять таймер канала можно с помощью ledc_bind_channel_timer().
    // Есть ещё низкоуровненвые функции, которые можно использовать для изменения параметров ШИМ: ledc_timer_set(), ledc_timer_rst(), ledc_timer_pause(), ledc_timer_resume().
    // Параметр структуры ledc_channel_config_t ledc_intr_type_t intr_type определяет запуск прерывания после завершения затухания. Для регистрации прерывания надо вызвать ledc_isr_register().

#endif /* MAIN_PWM_H_ */
