#include "BLEScanner.h"

BLEScanner::BLEScanner(std::vector<DeviceHolder>* devices) {
	this->addressList = new std::vector< uint64_t>;
	this->devices = devices;
}

void BLEScanner::startScan() {
	static std::mutex m;
	static std::mutex ma;
	static std::condition_variable cv;
	clock_t time = std::clock();

	BluetoothLEAdvertisementWatcher advWatcher;
	bool stopCalled = false;
	auto recvToken = advWatcher.Received([&time, this](BluetoothLEAdvertisementWatcher watcher, BluetoothLEAdvertisementReceivedEventArgs eventArgs) {
		BluetoothLEAdvertisement advertisement = eventArgs.Advertisement();
		IVector<BluetoothLEAdvertisementDataSection> dataSections = advertisement.DataSections();
		for (BluetoothLEAdvertisementDataSection dataSection : dataSections) {
			if (dataSection.DataType() == 9) {
				IBuffer data = dataSection.Data();
				if (data == NULL || data.Length() != 9) continue;
				char ch[9] = { 'K', 'a', 'm', 'i', 'z', 'e', 'l', 'k', 'a' };
				if (!memcmp(data.data(), ch, 9)){
					std::unique_lock l(ma);
					bool b = false;
					for (uint64_t u : *this->addressList) {
						if (u == eventArgs.BluetoothAddress()) {
							b = true;
							break;
						}
					}
					if (b == false) this->addressList->push_back(eventArgs.BluetoothAddress());
				}
			}
		}
		if ((std::clock() - time) / (clock_t)CLOCKS_PER_SEC >= 10) {
			watcher.Stop();
			std::unique_lock l(m);
			cv.notify_all();
		}
	});
	auto stoppedToken = advWatcher.Stopped([&stopCalled](BluetoothLEAdvertisementWatcher watcher, BluetoothLEAdvertisementWatcherStoppedEventArgs eventArgs) {
		std::unique_lock l(m);
		stopCalled = true;
		cv.notify_all();
	});
	{
		std::unique_lock l(m);
		advWatcher.Start();
		cv.wait(l, [&stopCalled, &advWatcher] {
			return stopCalled && (advWatcher.Status() == BluetoothLEAdvertisementWatcherStatus::Aborted
				|| advWatcher.Status() == BluetoothLEAdvertisementWatcherStatus::Stopped);
		});
	}
	// remove event handlers
	advWatcher.Received(recvToken);
	advWatcher.Stopped(stoppedToken);

	for (uint64_t addr : *this->addressList) {
		BluetoothLEDevice dev = BluetoothLEDevice::FromBluetoothAddressAsync(addr).get();
		if (NULL == dev)
			continue;
		GattDeviceServicesResult result = dev.GetGattServicesAsync(BluetoothCacheMode::Uncached).get();
		if (result.Status() == GattCommunicationStatus::Success) {
			IVectorView<GattDeviceService> services = result.Services();
			devices->push_back(DeviceHolder(addr));
			for (GattDeviceService service : services) {
				if (service.Uuid() != guid(DeviceHolder::dataServiceUUID) && service.Uuid() != guid(DeviceHolder::infoServiceUUID))
					continue;
				GattCharacteristicsResult charListR = getNested3([&service]() { return service.GetCharacteristicsAsync(); });
				IVectorView<GattCharacteristic> charList = charListR.Characteristics();
				for (GattCharacteristic characteristic : charList) {
					GattCharacteristicProperties charProp = characteristic.CharacteristicProperties();
					if (((int)charProp & (int)GattCharacteristicProperties::Read) == false)
						continue;
					IAsyncOperation<GattReadResult> valueResult = characteristic.ReadValueAsync();
					while (valueResult.Status() != AsyncStatus::Completed)
						continue;
					GattReadResult valueR = valueResult.GetResults();
					IBuffer value = valueR.Value();
					if (value == NULL || value.Length() == 0)
						continue;
					devices->back().setValues(characteristic.Uuid(), value.data(), value.Length());
				}
			}
		}
	}





}