﻿#include "DeviceListDialog.h"

DeviceListDialog::DeviceListDialog(std::vector<DeviceHolder>* devices, std::vector<uint64_t>* checkedDevices)
		: wxDialog(&MainWindow::getInstance(), wxID_ANY, wxT("Wybierz urządzenie"), wxDefaultPosition, wxSize(250, 350)) {
	this->checkedDevices = checkedDevices;
	this->tips = new std::map<wxWindowID, std::pair<wxCheckBox*, std::string>>;
	this->isTipExist = false;
	wxPanel* panel = new wxPanel(this, wxID_ANY);
	wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer* checkboxsizer = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer* downSizer = new wxBoxSizer(wxHORIZONTAL);
	wxButton* okButton = new wxButton(this, wxID_ANY, wxT("OK"), wxDefaultPosition, wxSize(70, 30));
	wxButton* closeButton = new wxButton(this, wxID_ANY, wxT("Zamknij"), wxDefaultPosition, wxSize(70, 30));
//	addressList->push_back(14454534344);
//	addressList->push_back(73283772728);
//	addressList->push_back(14958292839);
	for (DeviceHolder devH : *devices) {
		uint64_t addr = devH.bleAddress;
		wxCheckBox* cbox = new wxCheckBox(panel, wxID_ANY, AddrToString(addr));
		checkboxsizer->Add(cbox, 0, wxLEFT, 20);
//		cbox->Bind(wxEVT_ENTER_WINDOW, [&](wxMouseEvent& event) {
//			wxRichToolTip tp("1", "2");
//			tp.SetTimeout(0, 1500);
//			tp.ShowFor(cbox);
//			});
		cbox->Bind(wxEVT_ENTER_WINDOW, &DeviceListDialog::onMove, this);
	//	cbox->Bind(wxEVT_LEAVE_WINDOW, &DeviceListDialog::onLeave, this);
	//	cbox->Bind(wxEVT_ENTER_WINDOW, &DeviceListDialog::onLeaveCheckbox, this);
		//Connect(cbox->GetId(), wxEVT_MOVE, wxMoveEventHandler(DeviceListDialog::onMove));
		boxes.push_back(std::pair(cbox, addr));
		std::stringstream stream;
		stream << "Device name: " + devH.deviceName + "\nSerial number: " + devH.serialNumber + "\nDevice version: " + devH.deviceVersion
			+ "\nAuthor of idea: " + devH.author + "\nTechnical implementation: " + devH.technicalImplementation + "\nIs ready: " + devH.ready;
		this->tips->insert(std::pair(cbox->GetId(), std::pair(cbox, stream.str())));
	}
	panel->SetSizer(checkboxsizer);
	downSizer->Add(okButton, 1);
	downSizer->Add(closeButton, 1, wxLEFT, 5);
	sizer->Add(panel, 1);
	sizer->Add(downSizer, 0, wxALIGN_CENTER | wxTOP | wxBOTTOM, 10);
	this->SetSizer(sizer);
	Connect(closeButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(DeviceListDialog::onQuit));
	Connect(okButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(DeviceListDialog::onOk));
	Centre();
	ShowModal();
	Destroy();
}


void DeviceListDialog::onLeave(wxEvent& event) {
	std::cout << "onLeave" << std::endl;
	std::map<wxWindowID, std::pair<wxCheckBox*, std::string>>::iterator it = this->tips->find(event.GetId());
	if (it == this->tips->end())
		return;
	if (it->second.first->HasFocus()) {
		std::cout << "leave2" << std::endl;
	}
	else {
		std::cout << "set focus" << std::endl;
		it->second.first->SetFocus();
	}
}

DeviceListDialog::~DeviceListDialog(){
	delete tips;
}

void DeviceListDialog::onMove(wxEvent& event) {
	std::cout << ".";
	if (this->isTipExist)
		return;
	std::map<wxWindowID, std::pair<wxCheckBox*, std::string>>::iterator it = this->tips->find(event.GetId());
	if (it == this->tips->end())
		return;
	this->isTipExist = true;
	std::cout << it->second.first->GetChildren().size() << " before" << std::endl;
	std::string msg = it->second.second;
	wxRichToolTip* tip = new wxRichToolTip(wxT("INFO"), msg);
	tip->SetTimeout(0, 500);
	tip->ShowFor(it->second.first);
	wxWindowList tipWindow = it->second.first->GetChildren();

	auto a = tipWindow.GetLast()->GetData();
	a->Bind(wxEVT_DESTROY, &DeviceListDialog::onLeaveCheckbox, this);
	a->Bind(wxEVT_LEAVE_WINDOW, &DeviceListDialog::onLeave, this);
	a->Bind(wxEVT_COMMAND_SET_FOCUS, [&](wxEvent& event) {std::cout << "set focus" << std::endl; });

		

	std::cout << it->second.first->GetChildren().size() << " after" << std::endl;

	it->second.first->SetFocus();
}

void DeviceListDialog::onLeaveCheckbox(wxEvent& event) {
	this->isTipExist = false;
	std::cout << "destroy tip" << std::endl;
}

void DeviceListDialog::onOk(wxCommandEvent& WXUNUSED(event)) {
	for (std::pair<wxCheckBox*, uint64_t>& box : this->boxes){
		if (box.first->IsChecked()) {
			this->checkedDevices->push_back(box.second);
		}
	}
	this->Close();
}

void DeviceListDialog::onQuit(wxCommandEvent& WXUNUSED(event)) {
	this->Close();
}

std::string DeviceListDialog::AddrToString(uint64_t addr){
	std::stringstream ret;
	ret << std::hex << std::setfill('0')
		<< std::setw(2) << ((addr >> (5 * 8)) & 0xff) << ":"
		<< std::setw(2) << ((addr >> (4 * 8)) & 0xff) << ":"
		<< std::setw(2) << ((addr >> (3 * 8)) & 0xff) << ":"
		<< std::setw(2) << ((addr >> (2 * 8)) & 0xff) << ":"
		<< std::setw(2) << ((addr >> (1 * 8)) & 0xff) << ":"
		<< std::setw(2) << ((addr >> (0 * 8)) & 0xff);
	return ret.str();
}