﻿#include "MainWindow.h"

MainWindow& MainWindow::getInstance() {
	static MainWindow* mainWindow = new MainWindow();
	return *mainWindow;
}

MainWindow::MainWindow() : wxFrame(NULL, wxID_ANY, wxT("Wibraap"), wxDefaultPosition, wxSize(300, 500)) {
	std::cout << "2" << std::endl;
	std::cout << "6" << std::endl;
	this->devices = new std::vector<DeviceHolder>;
	this->keyboard = NULL;
	wxColour col1, col2;
	col1.Set(wxT("#4f5049"));
	col2.Set(wxT("#4B0082"));
	wxFont fontDeviceStatus = wxFont(20, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false);
	wxFont fontButtonText = wxFont(14, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false);

	wxPanel* mainPanel = new wxPanel(this, wxID_ANY);
	mainPanel->SetBackgroundColour(col1);
	wxPanel* textPanel = new wxPanel(mainPanel, wxID_ANY);
	textPanel->SetBackgroundColour(col2);

	std::cout << "5" << std::endl;
	// Device status text bar
	deviceStatus = new wxStaticText(textPanel, wxID_ANY, wxT("Urządzenie nie podłączone"), wxDefaultPosition, wxDefaultSize, wxALIGN_CENTER_HORIZONTAL | wxST_NO_AUTORESIZE);
	deviceStatus->SetBackgroundColour(col2);
	deviceStatus->SetForegroundColour(wxColour(wxT("#FFFFFF")));
	deviceStatus->SetFont(fontDeviceStatus);
	wxBoxSizer* deviceStatusTextSizer = new wxBoxSizer(wxVERTICAL);
	deviceStatusTextSizer->Add(deviceStatus, 1, wxALIGN_TOP | wxEXPAND | wxALL, 10);
	textPanel->SetSizer(deviceStatusTextSizer);

	// Buttons
	connectButton = new wxButton(mainPanel, wxID_ANY, wxT("Wyszukaj Kamizelkę"), wxPoint(20, 20));
	connectButton->SetFont(fontButtonText);
	openFileButton = new wxButton(mainPanel, wxID_ANY, wxT("Otwórz plik"), wxPoint(20, 20));
	openFileButton->SetFont(fontButtonText);
	microphoneButton = new wxButton(mainPanel, wxID_ANY, wxT("Mikrofon"), wxPoint(20, 20));
	microphoneButton->SetFont(fontButtonText);
	keyboardButton = new wxButton(mainPanel, wxID_ANY, wxT("Klawiatura"), wxPoint(20, 20));
	keyboardButton->SetFont(fontButtonText);

	wxBoxSizer* deviceStatusPanelSizer = new wxBoxSizer(wxVERTICAL);
	deviceStatusPanelSizer->Add(textPanel, 1, wxALIGN_TOP | wxEXPAND);
	deviceStatusPanelSizer->Add(connectButton, 1, wxEXPAND | wxALL, 20);
	deviceStatusPanelSizer->Add(openFileButton, 1, wxEXPAND | wxALL, 20);
	deviceStatusPanelSizer->Add(microphoneButton, 1, wxEXPAND | wxALL, 20);
	deviceStatusPanelSizer->Add(keyboardButton, 1, wxEXPAND | wxALL, 20);
	mainPanel->SetSizer(deviceStatusPanelSizer);
	std::cout << "4" << std::endl;

	Connect(connectButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainWindow::onClickConnectButton));
	Connect(openFileButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainWindow::onClickOpenFileButton));
	Connect(microphoneButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainWindow::onClickMicrophoneButton));
	Connect(keyboardButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainWindow::onClickKeyboardButton));
	Connect(this->GetId(), wxEVT_CLOSE_WINDOW, wxCloseEventHandler(MainWindow::CloseIt));

	Bind(wxEVT_SIZE, &MainWindow::onResize, this);

	Centre();
}


void MainWindow::onClickConnectButton(wxCommandEvent& WXUNUSED(event)) {
	deviceStatus->SetLabelText(wxT("Poczekaj 15 sekund"));
	this->warpText(this->GetSize().x);
	std::vector<DeviceHolder>* devices = new std::vector<DeviceHolder>;
	BLEScanner* blescanner = new BLEScanner(devices);
	blescanner->startScan();
	std::vector<uint64_t> checkedDevices;
	DeviceListDialog* dialog = new DeviceListDialog(devices, &checkedDevices);
	if (checkedDevices.size() == 0) {
		deviceStatus->SetLabelText(wxT("Urządzeń nie znaleziono"));
		return;
	}
	for (uint64_t i : checkedDevices) {
		std::wcout << i << std::endl;
	}
	deviceStatus->SetLabelText(wxString(wxT("Wybrano urządzeń: "))  + std::to_string(checkedDevices.size()));
}
void MainWindow::onClickOpenFileButton(wxCommandEvent& WXUNUSED(event)) {

}
void MainWindow::onClickMicrophoneButton(wxCommandEvent& WXUNUSED(event)) {

}
void MainWindow::onClickKeyboardButton(wxCommandEvent& WXUNUSED(event)) {
	if (keyboardButton->GetLabelText() == wxT("Klawiatura")) {
		keyboardButton->SetLabelText(wxT("Zamknij"));
		this->connectButton->Hide();
		this->openFileButton->Hide();
		this->microphoneButton->Hide();
		keyboard->SetPosition(this->GetPosition() + wxPoint(285, 0));
		keyboard->Show();
	}
	else {
		keyboardButton->SetLabelText("Klawiatura");
		this->connectButton->Show();
		this->openFileButton->Show();
		this->microphoneButton->Show();
		keyboard->Hide();
	}
}
void MainWindow::CloseIt(wxCloseEvent& event){
	std::cout << "close" << std::endl;
	this->DestroyChildren();
	Destroy();
}
void MainWindow::onResize(wxSizeEvent& event){
	const auto w = event.GetSize().GetWidth();
	this->warpText(w);
	event.Skip();
}
void MainWindow::warpText(int width) {
	//deviceStatus->SetLabel(wxT("Urządzenie nie podłączone"));
	deviceStatus->Wrap(width);
	Layout();
}
void MainWindow::initialize() {
	keyboard = new Keyboard(this->GetPosition() + wxPoint(285, 0));
}
bool MainWindow::Show(bool b) {
	if(keyboard == NULL)
		this->initialize();
	return wxFrame::Show(b);
}